#!/bin/bash
#
# Copyright (C) 2001 Caldera International, Inc.  All Rights Reserved.
#

# Store the path to the script
SCRIPTPATH=`dirname $0`

# Specify a postinstall log file
PILOG=/var/tmp/postinstall.log

DEBUGMODE=0

# must cd to root directory or /auto/cdrom wont umount 
# cd /


USERNAME=`id -un`
if [ "$USERNAME" != "root" ] ; then
    echo " "
    echo "You must be root to run this script"
    exit 1
fi

INITPATH="/etc/rc.d/init.d"

# Are we running on a Unixware/OpenUnix system?
if [ -e "/unixware/bin/sh" ]; then
	UNIXWARE=1
else
	UNIXWARE=0
fi

# License the GUI
/opt/lsb-caldera.com-volution/msg/bin/msglicense

COUNTRY=US ; export COUNTRY
FQDN=`hostname --fqdn`; export FQDN
if echo $FQDN | grep -q '.'
then
        ORG=`echo $FQDN | awk -F. '{print $(NF-1) "." $NF}'`
else
        ORG="$FQDN"
fi
export ORG

# the base suffix for the directory
# NOTE: this DN must be "normalized" (no spaces around commas) for use in ACLs
ROOT="o=$ORG,c=$COUNTRY"; export ROOT;

# the LDAP rootdn
ROOTDNBIND="uid=ldaproot,$ROOT"
# the LDAP rootdn password
if [ -c /dev/random ]; then
	ROOTDNPW=`head -c16 /dev/random | uuencode -m - | sed -n '2s/=*$//;2p' | sed -e '1,$s/\//X/g'`; 
	ROOTDNPWCRYPT=`/usr/sbin/slappasswd -s $ROOTDNPW`
	while echo $ROOTDNPWCRYPT | grep '[=\/]' > /dev/null 2>&1
	do
		ROOTDNPW=`head -c16 /dev/random | uuencode -m - | sed -n '2s/=*$//;2p' | sed -e '1,$s/\//X/g'`; 
		ROOTDNPWCRYPT=`/usr/sbin/slappasswd -s $ROOTDNPW`
	done
else 
	echo "ERROR: /dev/random not found!  Cannot generate ROOTDNPW..." >&2
	exit 1
fi
export ROOTDNPW

# Handle slapd.conf upgrades.
# If there is no /etc/ldap/slapd.conf file already...
if [ ! -e "/etc/ldap/slapd.conf" ]; then
	echo "Can not find an existing /etc/ldap/slapd.conf file!  You must" >$2
	echo "not have openldap installed.  Exiting..." >$2
	exit 1
else
	# If there IS an existing /etc/ldap/slapd.conf file, pick out the
	# value of rootpw.  Compare it to "secret", and if it doesn't
	# match, you have two slapd.conf files that need to be merged;
	# otherwise, you're dealing with a default slapd.conf file.
	TMPROOTPW=`awk '/^rootpw/ {print $2}' /etc/ldap/slapd.conf`
	if grep ^rootpw /etc/ldap/slapd.conf > /dev/null; then
		TMPROOTPW=`grep ^rootpw /etc/ldap/slapd.conf | \
			sed 's/^rootpw[ 	]*//'`
	fi
	if [ -n "$TMPROOTPW" ] && [ "$TMPROOTPW" != "secret" ]; then
		if grep ^suffix /etc/ldap/slapd.conf > /dev/null; then
			ROOT=`grep ^suffix /etc/ldap/slapd.conf | \
				sed 's/^suffix[ 	]*//' | \
				sed s/\"//g`
		fi
		if grep ^rootdn /etc/ldap/slapd.conf > /dev/null; then
			ROOTDNBIND=`grep ^rootdn /etc/ldap/slapd.conf | \
				sed 's/^rootdn[ 	]*//' | \
				sed s/\"//g`
		fi
		if ! echo $TMPROOTPW | grep '^{' > /dev/null; then
			ROOTDNPW=$TMPROOTPW
			ROOTDNPWCRYPT=$TMPROOTPW
		fi
	fi
	# If $DEBUGMODE is set to 1, show the values of the
	# variables before moving forward
	if [ "$DEBUGMODE" = "1" ]; then
		echo "BASE is set to $BASE"
		echo "ROOT is set to $ROOT"
		echo "ROOTDNBIND is set to $ROOTDNBIND"
		echo "ROOTDNPWCRYPT is set to $ROOTDNPWCRYPT"
		echo "CONFFILE is set to $CONFFILE"
	fi
fi



# The Messaging Server portion in the tree.
BASE="ou=Volution Messaging Server,$ROOT"; export BASE

# the bind DN for read access by postfix and pam_ldap
MSGUSER="msguser"; export MSGUSER
MSGUSERBIND="uid=$MSGUSER,$BASE"; export MSGUSERBIND
# the password for MSGUSERBIND
if [ -c /dev/random ]; then
	MSGUSERPW=`head -c16 /dev/random | uuencode -m - | sed -n '2s/=*$//;2p' | sed -e '1,$s/\//X/g'`
else 
	# This code will never be reached, but for the sake of
	# consistency...
	echo "ERROR: /dev/random not found!  Cannot generate MSGUSERPW..." >&2
	exit 1
fi
export MSGUSERPW
# the encrypted password for MSGUSERBIND
MSGUSERPWCRYPT="{CRYPT}`/opt/lsb-caldera.com-volution/msg/bin/msgencrypt $MSGUSERPW`"

# the bind DN for the MSG administrator
MSGADMIN="admin"; export MSGADMIN
MSGADMINBIND="uid=$MSGADMIN,ou=MSG Users,ou=$FQDN,$BASE"; export MSGADMINBIND
MSGADMINPW="admin"; export MSGADMINPW
MSGADMINPWCRYPT="{CRYPT}`/opt/lsb-caldera.com-volution/msg/bin/msgencrypt $MSGADMINPW`"


# fix up config files using the ldaphost utility.
/opt/lsb-caldera.com-volution/msg/bin/msgldaphost \
	--base="$BASE" \
	--rootdn="$ROOTDNBIND" \
	--admin="$MSGADMIN" \
	--admindn="$MSGADMINBIND" \
	--uid="$MSGUSER" \
	--dn="$MSGUSERBIND" \
	--pw="$MSGUSERPW" > $PILOG 2>&1

chmod 640 /etc/ldap/slapd.conf

# Now configure slapd
# Run external script slapdconf
$SCRIPTPATH/slapdconf \
        --base="$BASE" \
        --root="$ROOT" \
        --rootdn="$ROOTDNBIND" \
        --rootdnpw="$ROOTDNPWCRYPT" \
        --conffile=/etc/ldap/slapd.conf >> $PILOG 2>&1

# Fix the imapd.conf file
sed     -e "s/%ADMIN%/$MSGADMIN/" \
        /etc/imapd.conf > /var/tmp/imapd.conf
mv -f /var/tmp/imapd.conf /etc/imapd.conf
chmod 644 /etc/imapd.conf

# Handle default key and certificate files for cyrus and postfix
# Run external script installcerts
$SCRIPTPATH/installcerts

# add myhostname to the postfix main.cf file
grep ^myhostname /etc/postfix/main.cf > /dev/null || {
	echo "myhostname = `hostname -f`" >> /etc/postfix/main.cf
	# add postmaster to postfix main.cf file
	echo "error_notice_recipient = postmaster@$FQDN" >> /etc/postfix/main.cf
	echo 'bounce_notice_recipient = $error_notice_recipient' >> /etc/postfix/main.cf
	echo '2bounce_notice_recipient = $error_notice_recipient' >> /etc/postfix/main.cf
	echo 'delay_notice_recipient = $error_notice_recipient' >> /etc/postfix/main.cf
}

# re-compile the various postfix maps
/usr/sbin/postalias hash:/etc/postfix/aliases
MAPS="access canonical relocated transport virtual"
for f in $MAPS
do
        /usr/sbin/postmap hash:/etc/postfix/${f} >> $PILOG 2>&1
done

# edit cyrus init.d script to fix up our stuff:
# increase file descriptors for 2500 users, and increase the listen q to 256

file=/etc/rc.d/init.d/cyrus
tmp=/tmp/cyrus$$

if [ -f $file ]
then

awk '
BEGIN {
	found = 0;
}
/# up the file/ {
	found = 1;
}
/Starting/ {
	if (found == 0) {
	print "\
			# up the file descriptors for up to 2500 clients\n\
			if [ -f /proc/sys/fs/file-max ]\n\
			then\n\
				files=`cat /proc/sys/fs/file-max`\n\
				if [ $files -lt 100000 ]\n\
				then\n\
					echo 100000 > /proc/sys/fs/file-max\n\
				fi\n\
				unset files\n\
			fi"
	}
	print $0
	next
}
/\usr\/cyrus\/bin\/master &/ { gsub("&", "-l 256 &") ; print $0; next }
/.*/ { print $0; next }

' < $file > $tmp

cp $tmp $file
chmod 700 $file
rm -f $tmp

fi

unset file
unset tmp

# do it live too so they don't have to reboot
# up the file descriptors for up to 2500 clients
if [ -f /proc/sys/fs/file-max ]
then
	files=`cat /proc/sys/fs/file-max`
	if [ $files -lt 100000 ]
	then
		echo 100000 > /proc/sys/fs/file-max
	fi
	unset files
fi

# done editting cyrus init.d script

# fix up the ldap config to increase the timeout value
file=/etc/pam_ldap/auth_ldap.conf
tmp=/tmp/ldap$$

if [ -f $file ]
then

awk '
/^bind_timelimit/ { next }
/.*/ { print $0; next }
END { print "bind_timelimit 60" }
' < $file > $tmp

cp $tmp $file
rm -f $tmp

fi

unset file
unset tmp

# done editting ldap config file
# make sure our ldap directory is there.
if [ ! -d /var/ldap/openldap-ldbm ]
then
        mkdir -p -m 0700 /var/ldap/openldap-ldbm
fi

#start everything up
# 1. ldap
# 2. imap
# 3. postfix
# 4. apache (done below, after apache config)

[ -f $INITPATH/ldap    ] && $INITPATH/ldap    start
[ -f $INITPATH/cyrus   ] && $INITPATH/cyrus   start
[ -f $INITPATH/postfix ] && $INITPATH/postfix start

# Initialize the LDAP database
# Run external script initldapdb
# FIXME - If this has already been done, don't do it again!  See how to
# check...
$SCRIPTPATH/initldapdb \
        --root="$ROOT" \
        --base="$BASE" \
        --fqdn="$FQDN" \
        --rootdn="$ROOTDNBIND" \
        --rootdnpw="$ROOTDNPW" \
        --admindn="$MSGADMINBIND" \
        --dn="$MSGUSERBIND" \
        --uid="$MSGUSER" \
        --userpw="$MSGUSERPWCRYPT" \
        --admin="$MSGADMIN" \
        --adminpw="$MSGADMINPWCRYPT" \
        --ldiffile=/etc/opt/lsb-caldera.com-volution/msg/ldif.base >> $PILOG 2>&1

# create an inbox for $MSGADMIN
printf "$MSGADMINBIND\n$MSGADMINPW"|/opt/lsb-caldera.com-volution/msg/bin/msginboxcreate --name=$MSGADMIN

# Set up apache for the webgui
HTCONF="/etc/httpd/httpd.conf"
grep /opt/lsb-caldera.com-volution/msg/webgui "$HTCONF" > /dev/null 2>&1 || {
echo \
'# START VOLUTION MESSAGING SERVER CONFIGURATION - DO NOT MODIFY
Alias /msg "/opt/lsb-caldera.com-volution/msg/webgui/"
<Directory "/opt/lsb-caldera.com-volution/msg/webgui">
        Options Indexes MultiViews ExecCGI
        AllowOverride None Limit FileInfo
        Order allow,deny
        Allow from all
</Directory>
Alias /msghelp "/usr/share/doc/OpenLinux/"
<Directory "/usr/share/doc/OpenLinux">
        Options Indexes MultiViews
        AllowOverride None
        Order allow,deny
        Allow from all
</Directory>
# END VOLUTION MESSAGING SERVER CONFIGURATION' >> "$HTCONF"
}

# Stop/kill hotplugd and set it to never start again!
if [ -e /etc/sysconfig/daemons/hotplugd ]; then
        sed -e 's/ONBOOT=yes/ONBOOT=no/g' /etc/sysconfig/daemons/hotplugd > /tmp/hotplugd
        mv /tmp/hotplugd /etc/sysconfig/daemons
        /etc/rc.d/init.d/hotplugd stop
fi

# Fix the /etc/httpd/php.ini file - for session and client config data files
grep volution/msg/tmp /etc/httpd/php.ini > /dev/null 2>&1 || {
if [ -f /etc/httpd/php.ini ]; then
	# comment out the previous session save path, and add ours
	awk '
	/.*/ {
		if ($1 == "session.save_path") {
			print "; START VOLUTION MESSAGING SERVER CONFIGURATION - DO NOT MODIFY";
			print ";" $0;
			print "session.save_path          = /var/opt/lsb-caldera.com-volution/msg/tmp";
			print "; END VOLUTION MESSAGING SERVER CONFIGURATION - DO NOT MODIFY";
		} else {
			print $0;
		}
	}' /etc/httpd/php.ini > /var/tmp/php.ini
	cp /var/tmp/php.ini /etc/httpd/php.ini
	rm -f /var/tmp/php.ini
fi
}

# Make slapd and httpd start upon reboot
[ -f /etc/sysconfig/daemons/ldap ] && {
	sed -e 's/^ONBOOT=.*/ONBOOT="yes"/' /etc/sysconfig/daemons/ldap > \
		/var/tmp/ldap
	mv -f /var/tmp/ldap /etc/sysconfig/daemons/ldap
	chmod 644 /etc/sysconfig/daemons/ldap
	chown root:root /etc/sysconfig/daemons/ldap
}
[ -f /etc/sysconfig/daemons/httpd ] && {
	sed -e 's/^ONBOOT=.*/ONBOOT="yes"/' /etc/sysconfig/daemons/httpd > \
		/var/tmp/httpd1
	sed -e 's/^OPTIONS=.*/OPTIONS="-DSSL"/' /var/tmp/httpd1 > \
		/var/tmp/httpd2
	rm -f /var/tmp/httpd1
	mv -f /var/tmp/httpd2 /etc/sysconfig/daemons/httpd
	chmod 644 /etc/sysconfig/daemons/httpd
	chown root:root /etc/sysconfig/daemons/httpd
}

# LKP does not enable docview for startup at install time
[ -f /etc/sysconfig/daemons/docview ] && {
	sed -e 's/^ONBOOT=.*/ONBOOT="yes"/' /etc/sysconfig/daemons/docview > \
		/var/tmp/docview
	mv -f /var/tmp/docview /etc/sysconfig/daemons/docview
	chmod 644 /etc/sysconfig/daemons/docview
	chown root:root /etc/sysconfig/daemons/docview
}

# Re-start apache and docview
[ -f /usr/sbin/apachectl ] && {
	/usr/sbin/apachectl stop
	sleep 5
	/usr/sbin/apachectl startssl
}

[ -f /etc/rc.d/init.d/docview ] && {
	/etc/rc.d/init.d/docview stop
	sleep 5
	/etc/rc.d/init.d/docview start
}

# Set up FTP and the webserver for calendar support
CALDIR=/home/ftp/pub/calendar
HTHOME=/home/httpd/html
[ -d $CALDIR ] || {
	mkdir -p $CALDIR
	chown ftp:ftp $CALDIR
	chmod 755 $CALDIR
	[ -L $HTHOME/calendar ] || {
		ln -s $CALDIR $HTHOME/calendar
	}
	echo \
	"ftp    stream  tcp     nowait  root    /usr/sbin/tcpd  in.ftpd -l -a" \	> /etc/inet.d/wu-ftpd
	killall -HUP inetd
}

# add sievebuild to cron
crontab -u root -l > /tmp/tmpa$$
sed -e /sievebuild/d < /tmp/tmpa$$ > /tmp/tmpb$$
if [ -d /unixware ]
then
	echo "17,47 * * * * (cd /linux/bin; ./bash -c /opt/lsb-caldera.com-volution/msg/bin/msgsievebuild)" >> /tmp/tmpb$$
else
	echo "17,47 * * * * /opt/lsb-caldera.com-volution/msg/bin/msgsievebuild" >> /tmp/tmpb$$
fi
crontab -u root /tmp/tmpb$$
rm -f /tmp/tmpa$$ /tmp/tmpb$$

# configure webmail/mysql
/opt/lsb-caldera.com-volution/msg/bin/msgimpsetup

# make sure that postfix and cyrus users can resolve addresses
chmod 644 /etc/resolv.conf

# apply the maintenance pack
cd mpack-1.0.1-2
./install.sh
