%define _manprefix	%{_mandir}%{manEN}
%define _configdir	/var/imap
%define _partitiondir	/var/spool/imap
%define _sievedir	/usr/sieve
%define _cyrususer	cyrus
%define _cyrusgroup	mail

#
# Cyrus imapd
#
Summary:	Cyrus imapd
Name:		cyrus-imapd
Version:	2.0.13
Release:	3
Copyright:	CMU pubilc license
Group:		Applications
URL:		http://asg.web.cmu.edu/cyrus/imapd
Source0:	ftp://ftp.andrew.cmu.edu/pub/cyrus-mail/%{Name}-%{Version}.tar.gz
Source1:	cyrus.init
Patch0:		%{Name}-%{Version}-linux.patch
Patch1:		%{Name}-%{Version}-imtest.patch
Distribution:	Caldera, Inc.
Vendor:		Caldera, Inc. 
Packager:	Scott Simmons <simmons@caldera.com>
BuildRoot: 	%{_tmppath}/%{Name}-%{Version}
Requires:	cyrus-sasl >= 1.5.15
Conflicts:	imap, imap-devel

%Description
IMAP (Internet Message Access Protocol) is an Internet standards-track
protocol for accessing messages (mail, bboards, news, etc).  The Cyrus IMAP
server differs from other IMAP server implementations in that it is generally
intended to be run on sealed servers, where normal users are not permitted to
log in.  The mailbox database is stored in parts of the filesystem that are
private to the Cyrus IMAP system.  All user access to mail is through the IMAP,
POP3, or KPOP protocols. 

The private mailbox database design gives the server large advantages in
efficiency, scalability, and administratability.  Multiple concurrent
read/write connections to the same mailbox are permitted.  The server supports
access control lists on mailboxes and storage quotas on mailbox hierarchies. 

The Cyrus project is the implementation of IMAP servers by the Carnegie Mellon
University Computing Services Department.


%Prep
%setup
%patch0 -p1
%patch1 -p1

%Build
./configure \
	--with-com_err=/usr \
	--with-sasl=/usr/cyrus-sasl \
        --with-sasl-dir=/usr/cyrus-sasl/lib/sasl \
        --with-sasl-inc=/usr/cyrus-sasl/include \
	--with-openssl=/usr/lib \
	--prefix=%{_prefix} \
	--mandir=%{_manprefix} \
	--with-ucdsnmp=no
	
make depend
make all

%Install
%{mkDESTDIR}

make DESTDIR=$DESTDIR install

# we need the mkimap tool for postinstall (or put in all in this spec file)
install -m 755 tools/mkimap $DESTDIR%{_prefix}/bin/mkimap

# install the configuration files
install -d $DESTDIR/etc
install -m 644 master/conf/normal.conf $DESTDIR/etc/cyrus.conf
install -m 644 master/conf/imapd.conf $DESTDIR/etc/imapd.conf

#install the pam files
install -d $DESTDIR/etc/pam.d
install -m 644 master/conf/pam.conf $DESTDIR/etc/pam.d/imap
install -m 644 master/conf/pam.conf $DESTDIR/etc/pam.d/pop

install -d $DESTDIR%{SVIdir}
install -m755 %{SOURCE1} $DESTDIR%{SVIdir}/cyrus

#install documentation not covered in man pages
docdir=$DESTDIR%{_defaultdocdir}/%{Name}-%{Version}
install -d $docdir

%{fixUP} -T $DESTDIR%{SVIdir} -e 's:\@SVIdir\@:%{SVIdir}:'

%Post
# set up the SysV init links
/usr/lib/LSB/init-install cyrus

# currently, a "mail" group already exists on LTP-- but will
# need to add code to check and add "mail" group if needed

# add the cyrus user
/usr/sbin/useradd -c "Cyrus IMAP user" -f -1 -g %{_cyrusgroup} %{_cyrususer}

# add syslog logging
echo "local6.*						/var/log/imapd" >> /etc/syslog.conf
touch /var/log/imapd
chmod 600 /var/log/imapd
kill -SIGHUP `pidof syslogd`

# can we let the rpm do this instead?  but no cyrus user/group exists at that
# time?
# create the configdirectory
[ ! -d %{_configdir} ] && mkdir %{_configdir}
chown %{_cyrususer} %{_configdir}
chgrp %{_cyrusgroup} %{_configdir}
chmod 750 %{_configdir}

# create the partition directory
[ ! -d %{_partitiondir} ] && mkdir %{_partitiondir}
chown %{_cyrususer} %{_partitiondir}
chgrp %{_cyrusgroup} %{_partitiondir}
chmod 750 %{_partitiondir}

# create the sieve directory
[ ! -d %{sievedir} ] && mkdir %{_sievedir}
chown %{_cyrususer} %{_sievedir}
chgrp %{_cyrusgroup} %{_sievedir}
chmod 750 %{_sievedir}

# use the mkimap script to create the rest
su -c %{_prefix}/bin/mkimap cyrus

# OpenUnix does not use Linux filesystems, so running chattr on
# an OpenUnix/LKP installation will cause lots of ugly error 
# messages...  If we're on OpenUnix, don't run these commands.
if [ ! -e /unixware/bin/sh ]; then
   # set the user, quota, and partition directories to update synchronously--
   # this is necessary for linux systems using ext2 filesystems and failure
   # to do so may lead to data corruption and/or loss of mail after a system
   # crash
   chattr -R +S %{_configdir}/user
   chattr -R +S %{_configdir}/quota
   chattr -R +S %{_partitiondir}
   # also set the queue directory of the mail daemon to update synchronously
   # must know the MTA for that
   # chattr -R ???
fi

# add /etc/services entries
grep -q "^pop3[ 	]*110/tcp" /etc/services
if [ $? -eq 1 ]
then
	echo "pop3		110/tcp" >>/etc/services
fi
grep -q "^imap[ 	]*143/tcp" /etc/services
if [ $? -eq 1 ]
then
	echo "imap		143/tcp" >>/etc/services
fi
grep -q "^imsp[ 	]*406/tcp" /etc/services
if [ $? -eq 1 ]
then
	echo "imsp		406/tcp" >>/etc/services
fi
# this is in the doc-- but we don't need it do we?
#grep -q "^acap[ 	]*674/tcp" /etc/services
#if [ $? -eq 1 ]
#then
#	echo "acap		674/tcp" >>/etc/services
#fi
grep -q "^imaps[ 	]*993/tcp" /etc/services
if [ $? -eq 1 ]
then
	echo "imaps		993/tcp" >>/etc/services
fi
grep -q "^pop3s[ 	]*995/tcp" /etc/services
if [ $? -eq 1 ]
then
	echo "pop3s		995/tcp" >>/etc/services
fi
grep -q "^kpop[ 	]*1109/tcp" /etc/services
if [ $? -eq 1 ]
then
	echo "kpop		1109/tcp" >>/etc/services
fi
grep -q "^sieve[ 	]*2000/tcp" /etc/services
if [ $? -eq 1 ]
then
	echo "sieve		2000/tcp" >>/etc/services
fi
grep -q "^lmtp[ 	]*2003/tcp" /etc/services
if [ $? -eq 1 ]
then
	echo "lmtp		2003/tcp" >>/etc/services
fi
grep -q "^fud[ 	]*4201/tcp" /etc/services
if [ $? -eq 1 ]
then
	echo "fud		4201/tcp" >>/etc/services
fi

# reinitialize syslogd
kill -SIGHUP `pidof syslogd`

# start the master daemon
%{SVIdir}/cyrus start

# set up a sysconfig file 
cat << EOF > /etc/sysconfig/daemons/cyrus
IDENT=cyrus
DESCRIPTIVE="Cyrus IMAP Server"
ONBOOT="yes"
EOF

%Preun

# shut down master daemon
%{SVIdir}/cyrus stop >/dev/null

# remove the cyrus user
/usr/sbin/userdel cyrus


# remove the startup links
/usr/lib/LSB/init-remove cyrus

%Postun

# remove syslog logging
if [ -f /etc/syslog.conf ]
then
	grep -q "^local.*/var/log/imapd" /etc/syslog.conf
	if [ $? -eq 0 ]
	then
		grep -v "^local.*/var/log/imapd" /etc/syslog.conf >/tmp/syslog.restore
		cp /tmp/syslog.restore /etc/syslog.conf
		rm -f /tmp/syslog.restore
	fi
fi
rm -f /var/log/imapd

# reinitialize syslogd
kill -SIGHUP `pidof syslogd`


# remove the configdirectory
rm -rf %{_configdir}
rm -rf %{_partitiondir}
rm -rf %{_sievedir}

# don't need to chattr -S the cyrus directories since they are removed,
# but will need to chattr -S the postfix queue directories when we
# add the chattr +S of them to the spec file?

# not bothering to remove /etc/services entries for now


%Clean
%{rmDESTDIR}

%Files
%config(noreplace) /etc/cyrus.conf
%config(noreplace) /etc/imapd.conf
%config(noreplace) /etc/pam.d/imap
%config(noreplace) /etc/pam.d/pop
%config(noreplace) %{SVIdir}/cyrus
%doc COPYRIGHT doc/*.html
/usr/cyrus
/usr/include/cyrus
/usr/bin/*
/usr/lib/*
%{_manprefix}/man1/*
%{_manprefix}/man3/*
%{_manprefix}/man5/*
%{_manprefix}/man8/*
%{_manprefix}/perl5/man/man3/*
