#
# Copyright (C) 2001 Caldera International, Inc.  All Rights Reserved.
#
# Library routines and global defines for MSG backend scripts
#

use Getopt::Long;
use Net::Domain qw(hostname hostfqdn hostdomain);
use Net::LDAP qw(:all);                      # use for all code
use Net::LDAP::LDIF qw(:all);                # use for all code
use Net::LDAP::Util qw(ldap_error_name       
                       ldap_error_text) ;    # use for Error handling
use IO::Select;
use IO::File;


#
# Globals
#
$errmsg		= "";
$ldaphost	= "";
$ldapsuffix 	= "";
$rootdn  	= "";
$rootpw  	= "";
$msgadmin       = "";
$msgadmindn     = "";
$msgadminpw     = "";
$msgroot 	= "";
$msgbin  	= "";
$msgdn		= "";
$msguser	= "";
$msgpw		= "";
$msgtmp		= "";
$msgkey         = "";

#
# Functions
#
sub SetLocale {
    if ($_HASGETTEXT) {
        setlocale($_[0],$_[1]);
    } 
}
sub BindTextDomain {
    if ($_HASGETTEXT) {
        bindtextdomain($_[0],$_[1]);
    } 
}
sub TextDomain {
    if ($_HASGETTEXT) {
        textdomain($_[0]);
    } 
}
sub GetText {
    if ($_HASGETTEXT) {
        $_ = gettext($_[0]);
	return($_);
    } else {
        return($_[0]);
    }
}
sub getLDAPerrorString 
{
    my ($from,$mesg) = @_;
    $errmsg = "ERROR:$mesg->code:" .
	$from . " (" . ldap_error_name($mesg->code) . ") " . ldap_error_text($mesg->code);
    return $errmsg;
}
sub I_am_interactive {
        if (!system("/usr/bin/tty -s")) {
        	return 1
        } else {
        	return 0
        }
}
sub parse_conf {
  my $ldapport = "";

  open(FD, "/etc/opt/lsb-caldera.com-volution/msg//msg.conf") || die "ERROR:0:msg.conf:$!";
  @lines = <FD>;
  close(FD);

 
  foreach(@lines) {
        if (/^LDAPHOST:/) {
                @vals = split(":");
                $ldaphost = $vals[1];
                chomp($ldaphost);
        }
	if (/^LDAPPORT:/) {
                @vals = split(":");
                $ldapport = $vals[1];
                chomp($ldapport);
	}
        if (/^ROOTDN:/) {
                @vals = split(":");
                $rootdn = $vals[1];
                chomp($rootdn);
        }
        if (/^ROOTPW:/) {
                @vals = split(":");
                $rootpw = $vals[1];
                chomp($rootpw);
        }
        if (/^LDAPSUFFIX:/) {
                @vals = split(":");
                $ldapsuffix = $vals[1];
                chomp($ldapsuffix);
        }
        if (/^ADMIN:/) {
                @vals = split(":");
                $msgadmin = $vals[1];
                chomp($msgadmin);
        }
        if (/^ADMINDN:/) {
                @vals = split(":");
                $msgadmindn = $vals[1];
                chomp($msgadmindn);
        }
        if (/^ADMINPW:/) {
                @vals = split(":");
                $msgadminpw = $vals[1];
                chomp($msgadminpw);
        }
        if (/^ROOT:/) {
                @vals = split(":");
                $msgroot = $vals[1];
                chomp($msgroot);
        }
        if (/^BIN:/) {
                @vals = split(":");
                $msgbin = $vals[1];
                chomp($msgbin);
        }
        if (/^DN:/) {
                @vals = split(":");
                $msgdn = $vals[1];
                chomp($msgdn);
        }
        if (/^USER:/) {
                @vals = split(":");
                $msguser = $vals[1];
                chomp($msguser);
        }
        if (/^PW:/) {
                @vals = split(":");
                $msgpw = $vals[1];
                chomp($msgpw);
        }
        if (/^TMP:/) {
                @vals = split(":");
                $msgtmp = $vals[1];
                chomp($msgtmp);
        }
        if (/^VAROPT:/) {
                @vals = split(":");
                $msgvaropt = $vals[1];
                chomp($msgvaropt);
        }
  }

  # if test(hook) password is given we are done
  if ($msgadminpw ) {
	  return;
  }

  # Swap out the msgdn and set the msgadminpw for those of the user if
  # they were given to stdin.

  if (! -c *STDIN) {
	  ($udn, $upw, $ukey)  = <STDIN>;
	  if ( $udn ) {
		$msgadmindn = $udn;
		chomp($msgadmindn);
	  }
	  if ( $upw ) {
		$msgadminpw = $upw;
		chomp($msgadminpw);
	  }
	  if ( $ukey ) {
		$msgkey = $ukey;
		chomp($msgkey);
	  }
  }


  # If the password wasn't given to stdin, and rootpw is not set, then 
  # we need to prompt for a password.
  if ( !$msgadminpw ) {
  	my $tty = (IO::File->new('/dev/tty', O_RDWR) ||
                 *STDERR || *STDIN || *STDOUT);
  	$tty->autoflush(1);
  	$tty->print(GetText("Volution msg admin password: "));
  	my $ostty;
  	chomp($ostty = `stty -g`);
  	system("/bin/stty -echo -icanon min 1 time 0 2>/dev/null || " .  
		"/bin/stty -echo cbreak");
  	chomp($msgadminpw = $tty->getline);
  	$tty->print("\n");
  	system("/bin/stty $ostty");
  }
  # Append the ldap port name to the ldap host name if its set
  if ($ldapport) {
	$ldaphost = "$ldaphost:$ldapport";
  }
  if ($MSGDEBUG) {
	print "ldaphost   = $ldaphost\n";
        print "ldapsuffix = $ldapsuffix\n";
        print "rootdn     = $rootdn\n";
        print "rootpw     = secret \n";
  }
}

# check for syntactically correct email address.
# fairly restrictive here we require name@domain syntax.
sub validmailaddr {

	my $addr = @_[0];

	my @tmp = split('@', $addr);
	if (scalar(@tmp) != 2) {
		return(0);
	}
	my $name = $tmp[0];
	my $domain = $tmp[1];
	if ($name eq "") {
		return(0);
	}
	if  ($domain eq "") {
		return(0);
	}
	return(1);
}

# check for syntactically correct email address.
# this is the premissive version, if an @ is present then both parts
# must be there user@domain, otherwise anything goes
# checking for source routes, uucp, and % hacks is just way beyond what
# should do here.
sub relaxvalidmailaddr {

	my $addr = @_[0];

	if (index($addr, "@") == -1) {
		return(1);
	}
	my @tmp = split('@', $addr);
	if (scalar(@tmp) != 2) {
		return(0);
	}
	my $name = $tmp[0];
	my $domain = $tmp[1];
	if ($name eq "") {
		return(0);
	}
	if  ($domain eq "") {
		return(0);
	}
	return(1);
}

# we don't allow double quotes in certain arguments, namely mail addresses
sub noquote {
	my ($attr, $val) = @_;

	if (index($val, "\"") >= 0) {
		print STDERR "--$arg=$val contains invalid characters, no quotes please\n";
		exit(1);
	}
}

# call validmailaddr and print error and die if syntax error
sub checkaddrsyntax {
	my ($attr, $val) = @_;

	noquote($attr, $val);
	if (validmailaddr($val) == 0) {
		print STDERR "ERROR:256:syntax error in address: --", $attr, "=", $val, "\n";
		exit(1);
	}
}

# call relaxvalidmailaddr and print error and die if syntax error
sub relaxcheckaddrsyntax {
	my ($attr, $val) = @_;

	noquote($attr, $val);
	if (relaxvalidmailaddr($val) == 0) {
		print STDERR "ERROR:256:syntax error in address: --", $attr, "=", $val, "\n";
		exit(1);
	}
}

# this depends on a valid dn, parse and get the first attribute.
sub getfirstfromdn {
	my $dn = @_[0];
	my @tmp = split(',', $dn);
	my @first = split('=', $tmp[0]);
	my $val = $first[1];
	return $val;
}

#
# check that a path is valid, this is a syntax check and for now
# is just has to start with a '/'
sub checkpath {
	my ($arg, $val) = @_;

	if (substr($val, 0, 1) ne "/") {
		print STDERR "ERROR:256:--$arg=$val is not a fully qualified pathname\n";
		exit(1);
	}
}

# verify that argument length does not exceed the given value
sub arglengthcheck {
	my ($arg, $val, $maxlen) = @_;

	$len = length($val);

	if ($len > $maxlen) {
		print STDERR "ERROR:256:--$arg length exceeds maximum value of $maxlen\n";
		exit(1);
	}
}

# validate the character set for the given argument.
# currently the charset is limited to printable ASCII characters

sub charsetcheck {
	my ($arg, $val, $newline) = @_;

	$error = 0;
	$len = length($val);
	for ($index = 0; $index < $len; $index = $index + 1) {
		$char = substr($val, $index, 1);
		# newlines ok in one case.
		if ($newline) {
			if (ord($char) == 10) {
				next;
			}
		}
		# reject control characters
		if (ord($char) < 32) {
			$error = 1;
			print STDERR "offending character in decimal: ", ord($char), "\n";
			last;
		}
	}
	if ($error) {
		print STDERR "--$arg=$val contains invalid characters\n";
		exit(1);
	}
}

# backslash blows openldaps mind when used in a dn so we prevent this.
# perl ldap module has additional bugs that searching for ( or ) or *
# does not work even if properly escaped \( \) \*
# so we disallow all the characters in attributes that are searched for.
sub nobackslash {
	my ($arg, $val, $query) = @_;

	if (index($val, "\\") >= 0) {
		print STDERR "--$arg=$val contains invalid characters, no backslash please\n";
		exit(1);
	}
	if (index($val, "(") >= 0) {
		print STDERR "--$arg=$val contains invalid characters, no ( please\n";
		exit(1);
	}
	if (index($val, ")") >= 0) {
		print STDERR "--$arg=$val contains invalid characters, no ) please\n";
		exit(1);
	}
	if (index($val, "*") >= 0) {
		print STDERR "--$arg=$val contains invalid characters, no * please\n";
		exit(1);
	}
}

# this is our global argument checker
# is is concerned with an ever growing set of checks including
# mail address syntax, query syntax,
# and character set, and argument length.
sub argcheck {
	my ($arg, $val, $query) = @_;

	# only check arguments with a value
	if ($val eq "") {
		return;
	}
	$newline = 0;
	if ($arg eq "vacationmessage") {
		$newline = 1;
		arglengthcheck($arg, $val, 1024);
	} else {
		arglengthcheck($arg, $val, 128);
	}
	# make sure characterset is ok
	charsetcheck($arg, $val, $newline);
	if ($arg eq "uid") {
		nobackslash($arg, $val);
	}
	if ($arg eq "domain") {
		nobackslash($arg, $val);
	}
	if ($arg eq "name") {
		nobackslash($arg, $val);
	}

	# now syntax check any mail addresses
	# these ones are strict, just user@domain allowed
	if ($arg eq "mail") {
		checkaddrsyntax($arg, $val);
		nobackslash($arg, $val);
	}
	if ($arg eq "alternatemail") {
		checkaddrsyntax($arg, $val);
		nobackslash($arg, $val);
	}
	if ($arg eq "owner") {
		checkaddrsyntax($arg, $val);
	}
	if ($arg eq "member") {
		checkaddrsyntax($arg, $val);
	}
	# these ones could be looser source allowing source routes and UUCP.
	if ($arg eq "forward") {
		relaxcheckaddrsyntax($arg, $val);
	}
	if ($arg eq "remote") {
		relaxcheckaddrsyntax($arg, $val);
	}
	# pathname checks
	if ($arg eq "program") {
		checkpath($arg, $val);
	}
	if ($arg eq "file") {
		checkpath($arg, $val);
	}
}

# stuff for calling the setuid commands

sub
do_cmd
{
	my $cmd = @_[0];

	$rc = open(FD, $cmd);
	if ($rc <= 0) {
		return(0);
	}
	print(FD "$msgadmindn\n");
	print(FD "$msgadminpw\n");
	$rc = close(FD);
	if (!$rc) {
		return(0);
	}
	return(1);
}

sub
do_rootread
{
	my ($file, $aref) = @_;
	my $tmpfile = "$msgtmp/tmp$$";

	# clear the array
	splice(@$aref, 0);

	# now fill it up
	$cmd = "|$msgbin/msgroot --read=$file > $tmpfile\n";
	$ret = do_cmd($cmd);
	if ($ret == 0) {
		return($ret);
	}
	# go get our file contents
	if (!open(FD, $tmpfile)) {
		print("ERROR:256:Unable to open tmp file: $tmpfile\n");
		exit(1);
	}
	@$aref = <FD>;
	close(FD);
	unlink($tmpfile);
	return(1);
}

sub
do_rootwrite
{
	my ($file, $aref) = @_;
	my $tmpfile = "$msgtmp/tmp$$";

	# output the array to the tmpfile
	if (!open(FD, ">$tmpfile")) {
		print("ERROR:256:Unable to create tmp file: $tmpfile\n");
		exit(1);
	}
	foreach $line (@$aref) {
		print FD $line;
	}
	close(FD);

	# send it
	$cmd = "|$msgbin/msgroot --write=$file --from=$tmpfile";
	$ret = do_cmd($cmd);
	unlink($tmpfile);
	if ($ret == 0) {
		print("ERROR:256:Unable write failed $file\n");
		exit(1);
	}
	return(1);
}

sub
do_rootserviceaction
{
	my ($service, $action) = @_;

	# reload
	$cmd = "|$msgbin/msgroot --service=$service --action=$action";
	$ret = do_cmd($cmd);
	if ($ret == 0) {
		die("ERROR:256:$service $action failed");
	}
	return(1);
}

sub
do_rootbackupdb
{

	# Backup the cyrus mailboxes.db
	$cmd = "|$msgbin/msgroot --backupdb";
	$ret = do_cmd($cmd);
	if ($ret == 0) {
		die("ERROR:256:Backup of mailboxes.db failed");
	}
	return(1);
}

#
# filter builder for ldap queries to escape out special characters
# pass in the filter with %1, %2 prototypes in arg[0], and the successive
# arguments arg[1]... will be escaped and sustituted into the filter.
#
sub
fbuild
{
	$args = scalar(@_) - 1;
	$filter = $_[0];

	for ($index = $args; $index > 0; $index--) {
		$sub = "%$index";
		$arg = $_[$index];

		if (index($filter, $sub) < 0) {
			die("Internal error: $_[0] does not contain $sub");
		}

		$arg =~ s/\\/\\\\/g;
		$arg =~ s/\*/\\*/g;
		$arg =~ s/\(/\\(/g;
		$arg =~ s/\)/\\)/g;

		$filter =~ s/$sub/$arg/;
	}
	return($filter);
}

# check for duplicate array entry.
sub IsPresent {
	my ($item, @list) = @_;
	foreach (@list) {
		if ($item eq $_) {
			return 1;
		}
	}
	return 0;
}

# identify the primary domain
sub getmasterdomain {
	my @tmp = split(',', $msgadmindn);
	my $masterdomain = substr($tmp[2], 3, 9999);
	return $masterdomain;
}

# identify the postmaster mail address in a way that
# non-priviledged users can get to it,
# used to read main.cf but that requires root
sub getpostmaster {
	my $masterdomain = getmasterdomain();
	my $postmaster = "postmaster\@$masterdomain";
	return $postmaster;
}

return 1;	# required
