<?php
/*
 * Copyright (C) 2001 Caldera International, Inc.  All Rights Reserved.
 */

class Alias {
	/* Properties */

	// sv = single valued, mv = multivalued
	var $mail;	// sv, Email address (with domain), *required*
	var $name;      // sv, Email without the domain, forced value
	var $domain;	// mv, domain which the alias belongs to
	var $description; // sv, description of the alias purpose
	var $joinable;	// sv, boolean, non-owner can add and delete
	var $owners;	// mv, array of user mail addresses who are owners
	var $ouids;	// mv, array of user mail addresses who are owners
	var $programs;	// mv, array of programs called on deliver
	var $files;	// mv, array of files appended on deliver
	var $mgenerics;	// mv, array of mail addresses (receipients)
	var $musers;	// mv, array of mail addresses (receipients)
	var $muids;	// mv, array of mail addresses (receipients)
	var $maliases;	// mv, array of mail addresses alias receipients
	var $mremotes;	// mv, array of mail addresses for non-domain members

	/* Methods */

	function Alias() {
		$numArgs = func_num_args();

		switch ($numArgs) {
		  case 0:
			$this->_emptyAlias();
			break;
		  case 1:
			$arg1 = func_get_arg(0);
			if (is_array($arg1)) {
				$ret = $this->_createAlias($arg1);
				if (! $ret) {
					$this = "";
				}
			} else {
				$this->_fetchAlias($arg1); // mail or name
			}
			break;
		  case 2:
			$arg1 = func_get_arg(0); // mail or name
			$arg2 = func_get_arg(2); // array of attributes
			$this->_fetchAlias($arg1, $arg2);
			break;
		  default:
			$this = "";
			break;
		}

	}
	/* Methods */

	function _fetchAlias($mail, $argArray="") {
		global $session_domain;

		if ($mail == "") {
			$this = "";
			return;
		}

		$pos = strpos($mail, '@');
		if ($pos === false) {
			$mail .= '@' . $session_domain;
		}

		/* Seems to have lost it's purpose
		if ($argArray != "") {
			$attrs = "";
			foreach ($argArray as $attr) $attrs .= _attrOption($attr);
		} else {
			$attrs = " --all";
		}
		*/
		$attrs = ' --all';

		$cmd = new Command('msgaliaslist');
		$cmd->add_arg('--mail', $mail);
		$cmd->add_arg($attrs);
		$cmd->run();
		$lines = $cmd->getOutput();

		if ($lines == "") {
			$this = "";
			return;
		}

		$count = 0;
		while ($line = array_shift($lines)) {
			$line = trim($line);

			if ($line != "") {
				list($attr, $value) = split(":", $line, 2);
				switch ($attr) {
					case "mail" :
						$count++;
						$this->mail = $value;
						break;
					case "name" :
						$this->name = $value;
						break;
					case "domain" :
						$count++;
						$this->domain = $value;
						break;
					case "description" :
						$count++;
						$this->description = $value;
						break;
					case "member" :
						$count++;
						$savedValue = $value;
						break;
					case "memberalias" :
						$count++;
						$this->maliases[] = $savedValue;
						unset($savedValue);
						break;
					case "memberuid" :
						$count++;
						$this->muids[] = $value;
						$this->musers[] = $savedValue;
						unset($savedValue);
						break;
					case "remote" :
						$count++;
						$this->mremotes[] = $value;
						break;
					case "owner" :
						$count++;
						$savedValue = $value;
						break;
					case "owneruid" :
						$count++;
						$this->ouids[] = $value;
						$this->owners[] = $savedValue;
						unset($savedValue);
						break;
					case "file" :
						$count++;
						$this->files[] = $value;
						$this->files[] = $value;
						break;
					case "program" :
						$count++;
						$this->programs[] = $value;
						break;
					case "joinable" :
						$count++;
						if (strcasecmp($value, 'true') == 0) {
							$this->joinable = true;
						} else {
							$this->joinable = false;
						}
						break;
					default:
						break;
				}
			}
		}

		if (! $count > 0 ) {
			$this = "";
			return;
		}
	}

	function _emptyAlias() {
		; // nothing
	}

	function _createAlias($varArray) {
		global $LAST_ERROR, $session_domain;

		$cmd = new Command('msgaliascreate');
		$cmd->add_arg('--domain', $session_domain);
		$varArray['domain'] = $session_domain;
		unset($LAST_ERROR);

		while (list ($attr, $val) = each ($varArray)) {
			$skip = false;

			$val = trim($val);

			if ($val == "")  continue;

			$attr = str_replace("_", "", $attr);

			if (! $this->_validate($attr, $val)) continue;

			switch ($attr) {
				case "mail" :
					//ignore
					continue;
					break;
				case "name" :
				    $this->name = $val;
				    break;
				case "owner" :
				    $a = $this->_parseAddressList($val);
				    if (count($a[0]) < 1) {
					$GLOBALS['LAST_ERROR'][] = 
						'No valid owners where specified';
				    	if (count($a[1]) > 0) {
						$GLOBALS['LAST_ERROR'][] = 
						'The following are bad owners:';
						$GLOBALS['LAST_ERROR'][] = 
						join(" ", $a[1]);
					}
					return false;
				    }

				    $this->owners = $a[0];
				    foreach ($this->owners as $o) {
					$cmd->add_arg('--owner', $o);
				    }

				    if (count($a[1]) > 0) {
						$badowners[] = $a[1];
				    }

				    $skip = true;
				    break;
				case "mgenerics" :
				    $a = $this->_parseAddressList($val);
				    if (count($a[0]) < 1) {
					$GLOBALS['LAST_ERROR'][] = 
						'No valid members where specified';
				    	if (count($a[1]) > 0) {
						$GLOBALS['LAST_ERROR'][] = 
						'The following are bad members:';
						$GLOBALS['LAST_ERROR'][] = 
						join(" ", $a[1]);
					}
					return false;
				    }

				    $this->mgenerics = $a[0];
				    foreach ($this->mgenerics as $m) {
					$cmd->add_arg('--member', $m);
				    }
				    
				    if (count($a[1]) > 0) {
						$badmembers[] = $a[1];
				    }

				    $skip = true;
				    break;
				case "file":
				    $this->files = $val;
				    break;
				case "program":
				    $this->programs = $val;
				    break;
				case "joinable":
				    $val = strtoupper($val);
				    break;
				default:
				    break;
			}

			if (! $skip) {
				$opt = $this->_getOption($attr);
				if ($opt != "") {
					$cmd->add_arg($opt, $val);
				}
			}
		}

		if (! isset($this->name)) {
			$GLOBALS['LAST_ERROR'][] = 'No alias was specified.';
			$GLOBALS['LAST_ERROR'][] = 'Aliases require a value for the alias email address';
			return false;
		}
		if (! isset($this->owners)) {
			$GLOBALS['LAST_ERROR'][] = 'No owner was specified.';
			$GLOBALS['LAST_ERROR'][] = 'Aliases require at least one owner.';
			return false;
		}
		if (! (isset($this->mgenerics) || isset($this->files) || isset($this->programs))) {
			$GLOBALS['LAST_ERROR'][] = 'No receipients where specified.';
			$GLOBALS['LAST_ERROR'][] = 'Aliases require at least one receipient, either a member, file or program';
			return false;
		}

		$cmd->run();

		if ($cmd->getExitVal() != 0) {
			$GLOBALS['LAST_ERROR'] = $cmd->getErrors();
			$GLOBALS['LAST_ERROR'][] = "\non command:\n" . $cmd->concat();
			$GLOBALS['LAST_ERROR_EXIT'] = $cmd->getExitVal();
			return false;
		} elseif (isset($badmembers) || isset($badowners)) {
			if (isset($LAST_ERROR)) {
				array_unshift($LAST_ERROR, 'Alias was CREATED but the following error occured:');
			} else {
				$LAST_ERROR[] = 'Alias was CREATED but the following error occured:';
				if (isset($badmembers)) {
				$LAST_ERROR[] = 'Could not add the following members to the alias because they are not valid email addresses:';
				foreach ($badmembers as $bad) {
					$LAST_ERROR[] = $bad;
				}
				}
				if (isset($badowners)) {
				$LAST_ERROR[] = 'Could not add the following owners to the alias because they are not valid email addresses:';
				foreach ($badowners as $bad) {
					$LAST_ERROR[] = $bad;
				}
				}
			}
			$retval = false;
		} else {
		$LAST_ERROR[] = 'Unknown error.' . $this->mail;
			$retval = true;
		}

		unset($this->mgenerics);
		unset($this->owners);
		unset($this->programs);
		unset($this->files);
		unset($this->maliases);
		unset($this->musers);
		unset($this->mremotes);
		$this->_fetchAlias($this->name);
	
		return $retval;
	}

	function modify($varArray) {

		$cmd = new Command('msgaliasreplace');
		$cmd->add_arg('--mail', $this->mail);
		$current = get_object_vars($this);

		while (list ($attr, $val) = each ($varArray)) {

			$skip = false;

			$val = trim($val);

			$attr = str_replace("_", "", $attr);

			if ($current[$attr] == $val && $attr != 'joinable') {
				continue;
			}

			if (! $this->_validate($attr, $val)) continue;

			switch ($attr) {
				case "mail" :
				case "name" :
				case "domain" :
				case "owners" :
				case "programs" :
				case "files" :
				case "mgenerics" :
				case "maliases" :
				case "muids" :
				case "mremotes" :
				    continue; // none of these can be modified
					       // with this function
				case "joinable":
				    $val = strtoupper($val);
				    break;
				default: 
				    break;
			}

			if (! $skip) {
				$opt = $this->_getOption($attr);
				if ($opt != "") {
					$cmd->add_arg($opt, $val);
				}
			}
		}

		$cmd->run();

		if ($cmd->getExitVal() == 0) {
			$this = new Alias($this->mail);
			return true;
		}
		$GLOBALS['LAST_ERROR'] = $cmd->getErrors();
		$GLOBALS['LAST_ERROR'][] = "\non command:\n" . $cmd->concat();
		$GLOBALS['LAST_ERROR_EXIT'] = $cmd->getExitVal();
		return false;
	}

	function delete() {
	
		$cmd = new Command('msgaliasdelete');
		$cmd->add_arg('--mail', $this->mail);
		$cmd->add_arg('--recurse');
		$cmd->run();

		if ($cmd->getExitVal() == 0) {
			return true;
		}
		$GLOBALS['LAST_ERROR'] = $cmd->getErrors();
		$GLOBALS['LAST_ERROR'][] = "\non command:\n" . $cmd->concat();
		$GLOBALS['LAST_ERROR_EXIT'] = $cmd->getExitVal();
		return false;
	}

	function isRequired($attr) {
		switch ($attr) {
			case 'mail':
			case 'name':
			case 'owner':
			case 'member':
				$retval = true;
				break;
			case 'domain':
			case 'joinable':
			case 'description':
			case 'file':
			case 'program':
			default:
				$retval = false;
				break;
		}
		return $retval;
	}

	function _validate($attr, $value) {
		return true;
	}

	function _getOption($attr) {
		switch ($attr) {
			case "_mail" :
			case "mail" :
				$retval = '';
				break;
			case "name" :
			case "_name" :
				$retval = "--name";
				break;
			case "description":
			case "_description":
				$retval = "--description";
				break;
			case "member" :
			case "_member" :
			case "mgenerics" :
			case "_mgenerics" :
				$retval = "--member";
				break;
			case "_owner" :
			case "owner" :
				$retval = "--owner";
				break;
			case "_joinable" :
			case "joinable" :
				$retval = "--joinable";
				break;
			case "_remote" :
			case "remote" :
				$retval = "--remote";
				break;
			case "_file" :
			case "file" :
				$retval = "--file";
				break;
			case "_program" :
			case "program" :
				$retval = "--program";
				break;
			default:
				$retval = "";
		}

		return $retval;
	}

	function _alias3() {
		; //nothing;
	}

	
	function setAttr($attr, $val) {
		switch ($attr) {
			case "mail":
				$this->mail = $val;
				$this->name = 
					substr($val, 0, strpos($val, '@'));
				break;
			case "description":
				$this->description = $val;
				break;
			default:
				break;
		}
	}

	function getAttrs(&$attrArray) {
		$attrArray = array("name");
		return;
	}

	function setMail($value) {
		$this->mail = $value;
		$this->name = substr($value, 0, strpos($value, '@'));
	}

	function getMail() {
		return $this->mail;
	}

	function getName() {
		return $this->name;
	}

	function getDescription() {
		return $this->description;
	}

	function isOwner($mixed) {
		global $session_domain;

		$pos = strpos($mixed, '@');
		if ($pos === false) {
			$user = new User($mixed);
			if ($user == "") {
				$user = new User($mixed. '@' . $session_domain);
			}
			if ($user != "") {
				$address = $user->mail;
			}
		} elseif (substr($mixed, $pos, strlen($mixed)) == '@' . $session_domain) {
			$address = $mixed;
		}

		if (! isset($address)) {
			return false;
		}

		if (count(array_intersect($this->owners, array($address))) == 1) {
			return true;
		}
		return false;
		
	}

	function isJoinable() {
		return $this->joinable;
	}

	function isOwnerUID($uid) {
		$tmp = new User($uid);
		if (count(array_intersect($this->ouids, array($uid))) == 1) {
			return true;
		}
		return false;
	}

	function isMemberUID($uid) {
		if (count(array_intersect($this->muids, array($uid))) == 1) {
			return true;
		}
		return false;
		
	}

	function comparisonFunctionName($field, $direction) {
		switch($field) {
			case "name":
				$cmpFunction = "byAliasName";
				break;
			case "mail":
				$cmpFunction = "byAliasMail";
				break;
			case "description":
				$cmpFunction = "byAliasDescription";
				break;
			default:
				$cmpFunction = "byAliasName";
				break;
		}

		switch($direction) {
			case "a":
				$cmpFunction .= "Ascend";
				break;
			case "d":
				$cmpFunction .= "Decend";
				break;
			default:
				$cmpFunction .= "Acend";
				break;
		}
		return $cmpFunction;
	}

	function match($expression, $attr="") {
		if ($attr != "") {
			$retval = ereg($expression, $this->$attr);
		} else {
			if (eregi($expression, $this->name)) {
				$retval = true;
			} else {
				$retval = false;
			}
		}
		return $retval;
	}

	function validMailAddress($address) {
	
		//if (ereg('^[^\(\)\<\>\@\,\;\:\\\"\.\[\] ]+[.[^\(\)\<\>\@\,\;\:\\\"\.\[\]]]*@[^\(\)\<\>\@\,\;\:\\\"\.\[\] ]+[.[^\(\)\<\>\@\,\;\:\\\"\.\[\]]]+$', $address)) {
		if (ereg('^[a-zA-Z0-9_.]+@[a-zA-Z0-9_.]+$', $address)) {
			return true;
		}
		return false;
	}

	function _splitMembers($list) {
		if (ereg('"', $list)) {
			//$first = strstr($list, '"');
			//$second = strstr(substr($list, '"', );
			$quoted = '"Bob Jones"';
			$GLOBALS['LAST_ERROR'][] = 
				'Sorry, quoted addresses are not supported (e.g. ' . $quoted . ') in this release';
			$GLOBALS['LAST_ERROR_EXIT'] = "";
			return false;
		}

		$list   = strtr($list, ",\n\r\t\0", " ");
		$list   = ereg_replace("[\n\r]", ' ', $list);
		$list   = trim($list);

		$mgenerics = split("[ ]+", $list);
		$mgenerics = array_unique($mgenerics);
		
		return $mgenerics;
	}

	function _validMember($address) {
		global $session_domain;

		$local = false;
		$pos = strpos($address, '@');
		if ($pos === false) {
			$address = $address . '@' . $session_domain;
			$local = true;
		} elseif (substr($address, $pos, strlen($address)) == '@' . $session_domain) {
			$local = true;
		}

		if ($local) {
			// is it a user mail address
			$tmp = new User($address);
			if ($tmp == "") {
				// is it an alias mail address
				$tmp = new Alias($address);
			}

			if ($tmp == "") {
				return false;
			} else {
				return true;
			}
		} elseif ($this->validMailAddress($address)) {
			return true;
		}
		return false;
	}

	function _parseAddressList($list) {
		global $session_domain;

		$addresses = $this->_splitMembers($list);
		if ($addresses === false) {
			return false;
		}

		foreach ($addresses as $address) {
			$pos = strpos($address, '@');
			if ($pos === false) {
				$local = true;
			} elseif (substr($address, $pos, strlen($address)) == '@' . $session_domain) {
				$local = true;
			} else {
				$local = false;
			}

			if ($local) {
				// is it a user mail address
				$tmp = new User($address);
				if ($tmp == "") {
					$address = $address . '@' . $session_domain;
					$tmp = new User($address);
				}
				if ($tmp == "") {
					// is it an alias mail address
					$tmp = new Alias($address);
				}
				if ($tmp == "") {
					$bad[]  = "$address";
				} else {
					$good[] = $tmp->mail;
				}
				
			}  else {
				if ($this->validMailAddress($address)) {
					$good[] = "$address";
				} else {
					$bad[]  = "$address";
				}
			}
		}

		return array($good, $bad);
	}

	function addMember($member) {
		global $session_user;

		// can add self if alias is joinable
		if ($member == $session_user && $this->joinable) {
			return $this->addMembers($member);
		}

		// if not joinable or not the member, can add if admin
		if (isAdmin()) {
			return $this->addMembers($member);
		}

		// if not any of those, can add if the owner
		if ($this->isOwner($session_user)) {
			return $this->addMembers($member);
		}

		// can't do it
		unset($GLOBALS['LAST_ERROR'][]);
		$GLOBALS['LAST_ERROR'][] = 'Insufficient permission';
		$GLOBALS['LAST_ERROR'][] = 'You must be the alias owner or the adminstrator.';
		return false;
	}

	function removeMember($member) {
		global $session_user;

		// can add self if alias is joinable
		if ($member == $session_user && $this->joinable) {
			return $this->removeMembers($member);
		}

		// if not joinable or not the member, can add if admin
		if (isAdmin()) {
			return $this->removeMembers($member);
		}

		// if not any of those, can add if the owner
		if ($this->isOwner($session_user)) {
			return $this->removeMembers($member);
		}

		// can't do it
		unset($GLOBALS['LAST_ERROR'][]);
		$GLOBALS['LAST_ERROR'][] = 'Insufficient permission';
		$GLOBALS['LAST_ERROR'][] = 'You must be the alias owner or the adminstrator.';
		return false;
	}

	function addMembers($list) {
		global $session_domain;

		$addresses = $this->_splitMembers($list);
		if ($addresses === false) {
			return false;
		}

		$cmd = new Command('msgaliasadd');
		$cmd->add_arg('--loop');
		$cmd->add_arg('--mail', $this->mail);
		foreach ($addresses as $address) {
			$local = false;
			$pos = strpos($address, '@');
			if ($pos === false) {
				$local = true;
			} elseif (substr($address, $pos, strlen($address)) == '@' . $session_domain) {
				$local = true;
			}

			if ($local) {
				// is it a user mail address
				$tmp = new User($address);
				if ($tmp == "" && $pos === false) {
					$address = $address . '@' . $session_domain;
					$tmp = new User($address);
				}
				if ($tmp == "") {
					// is it an alias mail address
					$tmp = new Alias($address);
				}
	
				if ($tmp == "") {
					$GLOBALS['LAST_ERROR'][] = '"' . $address . '"' . ' is not a valid address.';
					$error = true;
				} else {
					$address = $tmp->mail;
					$cmd->add_arg('--member', $address);
				}
			} elseif ($this->validMailAddress($address)) {
				$cmd->add_arg('--member', $address);
			} else {
				$GLOBALS['LAST_ERROR'][] = '"' . $address . '"' . ' is not a valid address.';
				$error = true;
			}
		}

		if ($error) {

			$GLOBALS['LAST_ERROR_EXIT'] = "";
			return false;

		} else {

			$cmd->run();

			if ($cmd->getExitVal() == 0) {
				return true;
			}
			$GLOBALS['LAST_ERROR'] = $cmd->getErrors();
			$GLOBALS['LAST_ERROR'][] = "\non command:\n" . $cmd->concat();
			$GLOBALS['LAST_ERROR_EXIT'] = $cmd->getExitVal();
			return false;
		}
		return true;
	}

	function removeMembers($list) {
		global $session_domain;

		$error = false;

		if (ereg('"', $list)) {
			//$first = strstr($list, '"');
			//$second = strstr(substr($list, '"', );
			$quoted = '"Bob Jones"';
			$GLOBALS['LAST_ERROR'][] = 
				'Sorry, the quoted addresses are not supported (e.g. ' . $quoted . ') in this release';
			$GLOBALS['LAST_ERROR_EXIT'] = "";
			return false;
		}

		$list   = strtr($list, ",\n\r\t\0", " ");
		$list   = ereg_replace("[\n\r]", ' ', $list);
		$list   = trim($list);
		$addresses = split("[ ]+", $list);
	
		$cmd = new Command('msgaliasremove');
		$cmd->add_arg('--mail', $this->mail);

		foreach ($addresses as $address) {
			$local = false;
			
			$pos = strpos($address, '@');
			if ($pos === false) {
				$local = true;
			} elseif (substr($address, $pos, strlen($address)) == '@' . $session_domain) {
				$local = true;
			} else {
				// must be a remote address
				if (count(array_intersect($this->mremotes, array($address))) > 0) {
					$cmd->add_arg('--member', $address);
				} else {
					$GLOBALS['LAST_ERROR'][] = '"' . $address . '"' . ' is not an alias member.';
					$error = true;
				}
				continue;
			}

			// is it a user mail address
			$tmp = new User($address);
			if ($tmp == "") {
				$address = $address . '@' . $session_domain;
				$tmp = new User($address);
			}
			if ($tmp != "") {
				$address = $tmp->mail;
				if (count(array_intersect($this->musers, array($address))) > 0) {
					$cmd->add_arg('--member', $address);
				} else {
					$GLOBALS['LAST_ERROR'][] = '"' . $address . '"' . ' is not an alias member.';
					$error = true;
				}
				continue;
			}

			$tmp = new Alias($address);
			if ($tmp != "") {
				$address = $tmp->mail;
				if (count(array_intersect($this->maliases, array($address))) > 0) {
					$cmd->add_arg('--member', $address);
				} else {
					$GLOBALS['LAST_ERROR'][] = '"' . $address . '"' . ' is not an alias member.';
					$error = true;
				}
				continue;
			}

			$GLOBALS['LAST_ERROR'][] = '"' . $address . '"' . ' is not an alias member.';
			$error = true;
		}

		if ($error) {
			$GLOBALS['LAST_ERROR_EXIT'] = "";
			return false;
		} else {

			$cmd->run();

			if ($cmd->getExitVal() == 0) {
				return true;
			}
			$GLOBALS['LAST_ERROR'] = $cmd->getErrors();
			$GLOBALS['LAST_ERROR'][] = "\non command:\n" . $cmd->concat();
			$GLOBALS['LAST_ERROR_EXIT'] = $cmd->getExitVal();
			return false;
		}
		return true;
	}

	function addOwners($list) {
		global $session_domain;

		$addresses = $this->_splitMembers($list);
		if ($addresses === false) {
			return false;
		}

		$cmd = new Command('msgaliasadd');
		$cmd->add_arg('--mail', $this->mail);

		foreach ($addresses as $address) {
			$local = false;
			$pos = strpos($address, '@');
			if ($pos === false) {
				$local = true;
			} elseif (substr($address, $pos, strlen($address)) == '@' . $session_domain) {
				$local = true;
			}

			if ($local) {
				// is it a user mail address
				$tmp = new User($address);
				if ($tmp == "") {
					$address = $address . '@' . $session_domain;
					$tmp = new User($address);
				}	
				if ($tmp == "") {
					$GLOBALS['LAST_ERROR'][] = '"' . $address . '"' . ' is not a valid user.';
					$error = true;
				} else {
					$cmd->add_arg('--owner', $tmp->mail);
				}
			} else {
				$GLOBALS['LAST_ERROR'][] = '"' . $address . '"' . ' is not a valid user.';
				$error = true;
			}
		}
		if ($error) {

			$GLOBALS['LAST_ERROR_EXIT'] = "";
			return false;

		} else {

			$cmd->run();

			if ($cmd->getExitVal() == 0) {
				return true;
			}
			$GLOBALS['LAST_ERROR'] = $cmd->getErrors();
			$GLOBALS['LAST_ERROR'][] = "\non command:\n" . $cmd->concat();
			$GLOBALS['LAST_ERROR_EXIT'] = $cmd->getExitVal();
			return false;
		}
		return true;
	}

	function removeOwners($list) {
		global $session_domain;

		$error = false;

		if (ereg('"', $list)) {
			//$first = strstr($list, '"');
			//$second = strstr(substr($list, '"', );
			$quoted = '"Bob Jones"';
			$GLOBALS['LAST_ERROR'][] = 
				'Sorry, the quoted addresses are not supported (e.g. ' . $quoted . ') in this release';
			$GLOBALS['LAST_ERROR_EXIT'] = "";
			return false;
		}

		$list   = strtr($list, ",\n\r\t\0", " ");
		$list   = ereg_replace("[\n\r]", ' ', $list);
		$list   = trim($list);
		$addresses = split("[ ]+", $list);
	
		$cmd = new Command('msgaliasremove');
		$cmd->add_arg('--mail', $this->mail);

		foreach ($addresses as $address) {
			$local = false;
			
			$pos = strpos($address, '@');
			if ($pos === false) {
				$local = true;
			} elseif (substr($address, $pos, strlen($address)) == '@' . $session_domain) {
				$local = true;
			} else {
				// owners can't be remote
				$GLOBALS['LAST_ERROR'][] = '"' . $address . '"' . ' is not an alias member.';
				$error = true;
				continue;
			}

			// is it a user mail address
			$tmp = new User($address);
			if ($tmp == "") {
				$address = $address . '@' . $session_domain;
				$tmp = new User($address);
			}
			if ($tmp != "") {
				if (count(array_intersect($this->owners, array($tmp->mail))) == 1) {
					$cmd->add_arg('--owner', $tmp->mail);
				} else {
					$GLOBALS['LAST_ERROR'][] = '"' . $address . '"' . ' is not an alias owner.';
					$error = true;
				}
				continue;
			}

			$GLOBALS['LAST_ERROR'][] = '"' . $address . '"' . ' is not an alias owner.';
			$error = true;
		}

		if ($error) {
			$GLOBALS['LAST_ERROR_EXIT'] = "";
			return false;
		} else {

			$cmd->run();

			if ($cmd->getExitVal() == 0) {
				return true;
			}
			$GLOBALS['LAST_ERROR'] = $cmd->getErrors();
			$GLOBALS['LAST_ERROR'][] = "\non command:\n" . $cmd->concat();
			$GLOBALS['LAST_ERROR_EXIT'] = $cmd->getExitVal();
			return false;
		}
		return true;
	}

}

/*
 * Ideally these comparison functions would be encapsulated in the 
 * Alias class as class methods. Can't work out how to pass a class
 * method to the usort() function yet.
 */
function byAliasNameAscend($left, $right) {
	return(strcasecmp($left->name,$right->name));
}

function byAliasNameDecend($left, $right) {
	return(-1 * (byAliasNameAscend($left, $right)));
}

function byAliasMailAscend($left, $right) {
	return(strcasecmp($left->mail,$right->mail));
}

function byAliasMailDecend($left, $right) {
	return(-1 * (byAliasMailAscend($left, $right)));
}

function byAliasDescriptionAscend($left, $right) {
	return(strcasecmp($left->description,$right->description));
}

function byAliasDescriptionDecend($left, $right) {
	return(-1 * (byAliasDescriptionAscend($left, $right)));
}

?>
