<?php
/*
 * Copyright (C) 2001 Caldera International, Inc.  All Rights Reserved.
 */

// abstract final class
class Authenticate {


	function isAuthenticated() {
		global $session_user;

		if ($session_user != "") {
			return true;
		}
		return false;
	}

	function authorize($user, $passwd) {
		global $LAST_ERROR, $LDAP_OC_ADMIN, $LDAP_OC_USER, $session_dn;
		global $LDAPHost, $LDAPanonuser, $LDAPanonpw, $LDAPsuffix;

		if ($passwd === "") {	
			return false;
		}

		$ldapServer = $LDAPHost;
		$ldapServer .= ($LDAPPort) ? ":$LDAPPort" : '';
		$link = ldap_connect($ldapServer);

		if ($link == 0) {
			$LAST_ERROR[] = "Couldn't connect to the LDAP Server.";
			$LAST_ERROR[] = "Make sure that LDAP on $ldapServer is running.";
			return false;
		}

		$bind = @ldap_bind($link, $LDAPanonuser, $LDAPanonpw);

		if (! $bind) {
			$LAST_ERROR[] = "Couldn't bind to the LDAP Server.";
			return false;
		} else {
			unset($bind);
		}

		$attributes = array("cn", "objectclass");
		$result = ldap_search($link, $LDAPsuffix,
      			"(&" .
			   "(|(objectclass=$LDAP_OC_ADMIN)(objectclass=$LDAP_OC_USER))" .
			   "(uid=$user)" .
			")",
			$attributes);
		$dnid = ldap_first_entry($link, $result);	
		if (! $dnid) {
			// no ldap entry was found for $user
			return false;
		}
		$dn = ldap_get_dn($link, $dnid);
		$objectclasses = ldap_get_values($link, $dnid, 'objectclass');
		ldap_free_result($result);

		$retval = @ldap_bind($link, $dn, $passwd);
		ldap_close($link);

		// Save the LDAP dn for this user for later use
		if ($retval) {
			$session_dn = $dn;
		}

		return $retval;
	}

	function reauthorize($passwd) {
		global $LDAPHost, $session_dn;
		$ldapServer = $LDAPHost;
		$ldapServer .= ($LDAPPort) ? ":$LDAPPort" : '';
		

		$link = ldap_connect($ldapServer);
		$retval = @ldap_bind($link, $session_dn, $passwd);
		ldap_close($link);

		return $retval;
	}

	function getCurrentKey() {
		global $session_key;

		return $session_key;
	}

	function generateCryptKey() {
		$fp   = fopen("/dev/urandom", "r");
		$lkey = fread($fp, 8);
		fclose($fp);
		return substr(md5(base64_encode($lkey)), 0 , 8);
	}

	function decryptString($string, $key) {
        	$iv = 'wl22NTC=';
		//not yet... problem in backend
		//$string = base64_decode($string);

	        $td = mcrypt_module_open(MCRYPT_TripleDES, '',
			MCRYPT_MODE_CFB, '');
        	mcrypt_generic_init ($td, $key, $iv);
        	$string = mdecrypt_generic ($td, $string);
        	mcrypt_generic_end ($td);

		return $string;
	}

	function encryptString($string, $key) {
		$iv = 'wl22NTC=';

		$td = mcrypt_module_open (MCRYPT_TripleDES, '',
			MCRYPT_MODE_CFB, '');
		mcrypt_generic_init ($td, $key, $iv);
		$string = mcrypt_generic ($td, $string);
		mcrypt_generic_end ($td);
		//not yet... problem in backend
		//$string = base64_encode($string);

		return $string;
	}

	function recryptPassword() {
		global $session_key, $session_passwd;

		// decoded here until backend is fixed
		$tmp = base64_decode($session_passwd);
		$passwd = Authenticate::decryptString($tmp, $session_key);
		return Authenticate::encryptPassword($passwd);
	}

	function encryptPassword($passwd) {
		global $session_passwd, $session_key;

		// get new key and encrypt passwd
		$tmpkey = Authenticate::generateCryptKey();
		$tmppasswd = Authenticate::encryptString($passwd, $tmpkey);

		// save the recrypted passwd and
		// send new session key to browser

		// encoded here until backend is fixed
		$session_passwd = base64_encode($tmppasswd);
		$session_key = $tmpkey; 
		header("Set-Cookie: session_key=$tmpkey; path=/");

		return true;
	}

	function clearCryptKey() {
		global $session_key;
		return;
	}


	function isAdmin() {
		$numArgs = func_num_args();

		switch ($numArgs) {
		  case 0:
			$retval =  Authenticate::_isAdmin0();
			break;
		  case 1:
			$uid = func_get_arg(0);
			$retval =  Authenticate::_isAdmin1($uid);
			break;
		  default:
			$retval = false;
			break;
		}

		return $retval;
	}

	function _isAdmin0() {
		global $session_user, $session_class;

		switch ($session_class) {
			case 'admin' :
				$retval = true;
				break;
			case 'user' :
				$retval = false;
				break;
			default:
				$retval = Authenticate::_isAdmin1($session_user);
				$session_class = $retval ? 'admin' : 'user';
				break;
		}

		return $retval;
	}

	function _isAdmin1($uid) {
		// Check and save admin status
		$cmd = new Command('msguserisadmin');
		$cmd->add_arg('--uid', $uid);
		$cmd->run();
		$lines = $cmd->getOutput();
		if (strcasecmp(trim($lines[0]), "true") == 0) {
			return true;
		} else {
			return false;
		}
	}

	function isAdminAlias($mail) {
		if ($mail == "postmaster") {
			return true;
		}
		return false;
	}

	function getSessionFile() {
		return $GLOBALS['TMPDIR'] . 'sess_' . $GLOBALS['PHPSESSID'];
	}

	function isAllowed($action) {
		global $session_user;

		switch ($action) {
			case 'changeprofile':
				$retval = true;
				break;
			case 'createalias':
				$retval = true;
				break;
			case 'changepw':
				$retval = true;
				break;
			default:
				$retval = false;
				break;
		}
		return $retval;
	}

	function changepw() {
		global $passthru_user, $passthru_newpw, $passthru_oldpw,
		       $session_user,  $session_passwd, $session_key;

		if (! Authenticate::isAdmin($session_user)) {
			$tmppw = Authenticate::decryptString($passthru_oldpw, $session_key);
			if (! Authenticate::reauthorize($tmppw)) {
				unset($GLOBALS['LAST_ERROR']);
				$GLOBALS['LAST_ERROR'][] = "Old password is invalid";
				return false;
			}
		}

		$cmd = new Command('msguserpw');
		$cmd->add_arg('', $passthru_user);
		$cmd->add_arg(Authenticate::getSessionFile());

		$cmd->run();

		if ($cmd->getExitVal() == 0) {
			if ($session_user == $passthru_user) {
				$session_passwd = base64_encode($passthru_newpw);
			}
			return true;
		}
		$GLOBALS['LAST_ERROR'] = $cmd->getErrors();
		$GLOBALS['LAST_ERROR'][] = "\non command:\n" . $cmd->concat();
		$GLOBALS['LAST_ERROR_EXIT'] = $cmd->getExitVal();
		return false;
	}
}

?>
