<?php
/*
 * Copyright (C) 2001 Caldera International, Inc.  All Rights Reserved.
 */

class Domain {
	
	var $domain;
	var $description;
	var $nusers;
	var $naliases;


	function Domain() {

		$numArgs = func_num_args();

		switch ($numArgs) {
		  case 0:
			$this->_emptyDomain();
			break;
		  case 1:
			$arg1 = func_get_arg(0);
			if (is_array($arg1)) {
				$ret = $this->_createDomain($arg1);
				if (! $ret) {
					$this = "";
				}
			} else {
				$this->_fetchDomain($arg1);
			}
			break;
		  case 2:
			$arg1 = func_get_arg(0); // uid
			$arg2 = func_get_arg(2); // array of attributes
			$this->_domain3($arg1, $arg2);
		  case 3:
			$arg1 = func_get_arg(0);
			$arg2 = func_get_arg(1);
			$arg3 = func_get_arg(2);
			$this->_domain3($arg1, $arg2, $arg3);
			break;
		  default:
			$this = "";
			break;
		}

	}

	function _emptyDomain() {
		; // nothing
	}

	function _fetchDomain($domain) {
		$cmd = new Command('msgdomaincount');
		$cmd->add_arg('--domain', $domain);
		$cmd->run();
		$lines = $cmd->getOutput();

		if ($lines != "") foreach ($lines as $line) {
			$line = trim($line);

			if ($line != "") {
				list($attr, $value) = split(":", $line, 2);

				switch ($attr) {
					case "ou" :
					case "domain" :
						$this->domain = $value;
						break;
					case "description" :
						$this->description = $value;
						break;
					case "nusers" :
						$this->nusers = $value;
						break;
					case "naliases" :
						$this->naliases = $value;
						break;
					default:
						break;
				}
			}
		}
	}
		
	function _domain3($domain = "", $nusers = "", $naliases = "") {
		$this->domain = $domain;
		$this->nusers = $nusers;
		$this->naliases = $naliases;
	}

	function _createDomain($varArray) {

		$cmd = new Command('msgdomaincreate');

		while (list ($attr, $val) = each ($varArray)) {

			$val = trim($val);

			if ($val == "")  continue;

			$attr = str_replace("_", "", $attr);

			if (! $this->_validate($attr, $val)) continue;

			switch ($attr) {
				case "domain" :
				    $val = strtolower($val);
				    $this->domain = $val;
				    break;
				default:
				    break;
			}

			$opt = $this->_getOption($attr);
			if ($opt != "") {
				$cmd->add_arg($opt, $val);
			}
		}

		$cmd->run();

		if ($cmd->getExitVal() == 0) {
			unset($GLOBALS['session_domainList']);
			$this->_fetchDomain($this->domain);
			return true;
		}
		$GLOBALS['LAST_ERROR'] = $cmd->getErrors();
		$GLOBALS['LAST_ERROR'][] = "\non command:\n" . $cmd->concat();
		$GLOBALS['LAST_ERROR_EXIT'] = $cmd->getExitVal();
		return false;
	}

	function _validate($attr, $val) {
		return true;
	}

	function _getOption($attr) {
		switch ($attr) {
			case "_domain" :
			case "domain" :
				$retval = "--domain";
				break;
			case "_description" :
			case "description" :
				$retval = "--description";
				break;
			default:
				$retval = "";
		}

		return $retval;
	}

	function delete() {
		$cmd = new Command('msgdomaindelete');
		$cmd->add_arg('--domain', $this->domain);
		$cmd->run();

		if ($cmd->getExitVal() == 0) {
			unset($GLOBALS['session_domainList']);
			return true;
		}
		$this->errorMsgs = $cmd->getErrors();
		return false;
	}

	function setDomain($domain) {
		$this->domain = $domain;
	}

	function setNUsers($count) {
		$this->nusers = $count;
	}

	function setNAliases($count) {
		$this->naliases = $count;
	}

	function getDomain() {
		return $this->domain;
	}

	function getNUsers() {
		return $this->nusers;
	}

	function getNAliases() {
		return $this->naliases;
	}

	function setAttr($attr, $value) {
		switch ($attr) {
			case "domain":
				$this->domain = $value;
				break;
			case "description":
				$this->description = $value;
				break;
			case "nusers":
				$this->nusers = $value;
				break;
			case "naliases":
				$this->naliases = $value;
				break;
			default:
				break;
		}
	}

	function getAttr($attr) {
		switch ($attr) {
			case "domain":
				$retval = $this->domain;
				break;
			case "description":
				$retval = $this->description;
				break;
			case "nusers":
				$retval = $this->nusers;
				break;
			case "naliases":
				$retval = $this->naliases;
				break;
			default:
				$retval = "";
				break;
		}
		return $retval;
	}

	function isRequired($attr) {
		switch ($attr) {
			case 'domain':
				$retval = true;
				break;
			case 'description':
			default:
				$retval = false;
				break;
		}
		return $retval;
	}

	function comparisonFunctionName($field, $direction) {
		switch($field) {
			case "domain":
				$cmpFunction = "_domain_byDomain";
				break;
			case "nusers":
				$cmpFunction = "_domain_byNUsers";
				break;
			case "naliases":
				$cmpFunction = "_domain_byNAliases";
				break;
			default:
				$cmpFunction = "_domain_byDomain";
				break;
		}

		switch($direction) {
			case "a":
				$cmpFunction .= "Ascend";
				break;
			case "d":
				$cmpFunction .= "Decend";
				break;
			default:
				$cmpFunction .= "Ascend";
				break;
		}
		return $cmpFunction;
	}

	function match($expr, $attr="") {
		if ($attr != "") {
			$retval = ereg($expression, $this->$attr);
		} else {
			if (	eregi($expression, $this->domain)      ||
				eregi($expression, $this->description) ) {
				$retval = true;
			} else {
				$retval = false;
			}
		}
		return $retval;
	}
}

/*
 * Ideally these comparison functions would be encapsulated in the 
 * User class as class methods. Can't work out how to pass a class
 * method to the usort() function yet.
 */
function _domain_byDomainAscend($left, $right) {
	return(strcasecmp($left->domain,$right->domain));
}

function _domain_byDomainDecend($left, $right) {
	return(-1 * (_domain_byDomainAscend($left, $right)));
}
				$cmpFunction = "_domain_byNUsers";

function _domain_byNUsersAscend($left, $right) {
	if ($left->nusers < $right->nusers) {
		return -1;
	} elseif ($left->nusers > $right->nusers) {
		return 1;
	} else {
		// must be ==
		return 0;
	}
}

function _domain_byNUsersDecend($left, $right) {
	return(-1 * (_domain_byNUsersAscend($left, $right)));
}

function _domain_byNAliasesAscend($left, $right) {
	if ($left->naliases < $right->naliases) {
		return -1;
	} elseif ($left->naliases > $right->naliases) {
		return 1;
	} else {
		// must be ==
		return 0;
	}
}

function _domain_byNAliasesDecend($left, $right) {
	return(-1 * (_domain_byNAliasesAscend($left, $right)));
}

?>
