<?php
/*
 * Copyright (C) 2001 Caldera International, Inc.  All Rights Reserved.
 */

class DomainList extends ObjectTable {

	function DomainList($varArray, $id) {

		unset($this->list);

		$cmd = new Command('msgdomaincount');
		$cmd->run();
		$lines = $cmd->getOutput();

		if ($lines != "") foreach ($lines as $line) {

			$line = trim($line);

			if ($line == "") {
				if ($domain != "") {
					$this->list[] = $domain;
					unset($domain);
				}
				continue;
			}

			if (! isset($domain)) {
				$domain = new Domain();
			}
	
			list($attr, $value) = split(":", $line, 2);
			$domain->setAttr($attr, $value);
		}

		// Setup the table
		if (Table::isActive($varArray, $id)) {
			$table = Table::getActive();
			$table->setParams($varArray, $this->count());
		} else {
			$table = new Table($id);

			$name = getMessage('MSG_DOMAIN_HEADING_DOMAIN');
			$table->addDataColumn($name, Column::ASCEND(), 'domain', "return Go('domainview.php?domain=%%ROWID%%')");

			$name = getMessage('MSG_DOMAIN_HEADING_NUSERS');
			$table->addDataColumn($name, Column::ASCEND(), 'nusers');

			$name = getMessage('MSG_DOMAIN_HEADING_NALIASES');
			$table->addDataColumn($name, Column::ASCEND(), 'naliases');

			$name = getMessage('MSG_USERS_HEADING_DELETE');
			$table->addDisplayColumn($name, "return Go('domaindelete.php?apath=a&tdomain=%%ROWID%%')");

			$table->setParams($HTTP_GET_VARS, $this->count());
			$table->setSortedColumn($table->firstSortedColumn());
		}

		// Apply sorting
		$attr = $table->getSortAttrName($table->getSortedColumn());
		$dir  = $table->getColumnSortDir($table->getSortedColumn());
		$this->sort(Domain::comparisonFunctionName($attr ,$dir));

		// Save table
		$this->table = $table;
		$this->table->save();
	}

	function getDomainName($index) {
		return $this->list[$index]->getDomain();
	}

	function getDomainAliasCount($index) {
		return $this->list[$index]->getNAliases();
	}

	function getDomainUserCount($index) {
		return $this->list[$index]->getNUsers();
	}

	function countDomains() {
		return count($this->list);
	}

	function addDomain($domain) {
		$this->list[] = $domain;
	}

	function domainExists($domain) {
		for ($i = 0; $i < count($this->list); $i++) {
			if ($this->list[$i] == $domain) {
				return true;
			}
		}
		return false;
	}

	function getTNUsers() {
		$retval = 0;
		for ($i=0; $i < $this->count(); $i++) {
			$retval += $this->list[$i]->nusers;
		}
		return $retval;
	}

	function getTNAliases() {
		$retval = 0;
		for ($i=0; $i < $this->count(); $i++) {
			$retval += $this->list[$i]->naliases;
		}
		return $retval;
	}

	function getListValue($index, $col="") {
		// if no $col then return the row value (unique id)
		if ($col == "") return $this->list[$index]->getDomain();

		switch ($col) {
			case '0':
			case 'domain':
				$retval = $this->list[$index]->getDomain();
				break;
			case '1' :
			case 'nusers':
				$retval = $this->list[$index]->getNUsers();
				break;
			case '2' :
			case 'naliases':
				$retval = $this->list[$index]->getNAliases();
				break;
			case '3' :
				$retval = $this->table->getTemplate('deleteIcon');
				$retval = stripslashes($retval);
				break;
			default:
				$retval = "";
				break;
		}

		return $retval;
	}
}

class DomainDropList {
	var $list;

	function DomainDropList() {
		global $session_domainList;
		if (isset($session_domainList)) {
			$this->list = $session_domainList;
		} else {
			$cmd = new Command('msgdomainlistall');
			$cmd->run();
			$lines = $cmd->getOutput();

			if ($lines != "") foreach ($lines as $line) {
				if ($line == "") continue;
				list($attr, $domain) = split(":", $line, 2);
				if ($attr == 'domain') {
					$this->list[] = trim($domain);
				}
			}
			sort($this->list);
			$this->save();
			//$tmp = new DomainList(array(''), 'domains:0');
			//$tmp->save();
			//$this->list = $session_domainList;
		}
	}

	function optionList() {
		global $session_domain;
		$retval = "";
		foreach ($this->list as $dom) {
			$retval .= '<option value="' . $dom . '"';
			if ($dom == $session_domain) {
				$retval .= ' selected="1"';
			}
			$retval .= '>';
			$retval .= $dom;
			$retval .= '</option>' . "\n";
		}

		return $retval;
	}

	function save() {
		global $session_domainList;

		if (! $session_domainList) {
			session_register('session_domainList');
		}

		$session_domainList = $this->list;
	}

	function unsave() {
		global $session_domainList;

		if (isset($session_domainList)) {
			session_unset('session_domainList');
			session_unregister('session_domainList');
		}
	}

	function destroy () {
		$this->unsave();
		$this = "";
	}


	function disable() {
		global $_NN;

		$retval  = "<script>\n";
		$retval .= "<!-- \n";
		if ($_NN) {
			//$retval .= "top.topframe.DomDisable();";
			$retval .= ';';
		} else {
			$retval .= "document.domainSelect.s_domain.disabled = 1";
		}
		$retval .= "\n// --> \n";
		$retval .= "</script>\n";
		
		return $retval;
	}

	function isDomain($domain) {
		foreach ($this->list as $d) {
			if ($d == $domain) return true;
		}
		return false;
	}
}

?>
