<?php
/*
 * Copyright (C) 2001 Caldera International, Inc.  All Rights Reserved.
 */

class Service {
	var $name;	// sv, service short name
	var $description; // sv, descriptive name of service
	var $pid;	// sv, process id for service
	var $command;	// sv, command value of ps 
	var $status;    // sv, status of command (e.g. UP, DOWN)
	var $actions;   // mv, array of action names supported (based on status)

	function Service() {
		$numArgs = func_num_args();

		switch ($numArgs) {
		  case 0:
			$this->_emptyService();
			break;
		  case 1:
			$arg1 = func_get_arg(0); // service name
			$this->_fetchService($arg1); 
			break;
		  default:
			$this = "";
			break;
		}

	}

	function _fetchService($name) {
		if ($name == "") {
			$this = "";
			return;
		}

		$cmd = new Command('msgservicelist');
		$cmd->add_arg('--service', $name);
		$cmd->add_arg('--all');
		$cmd->run();
		$lines = $cmd->getOutput();

		if ($lines == "") {
			$this = "";
			return;
		}

		$count = 0;
		foreach ($lines as $line) {
			$line = trim($line);

			if ($line != "") {
				list($attr, $value) = split(":", $line, 2);

				switch ($attr) {
					case "service" :
						$count++;
						$this->name = $value;
						break;
					case "description" :
						$count++;
						$this->description = $value;
						break;
					case "pid" :
						$count++;
						$this->pid = $value;
						break;
					case "command" :
						$count++;
						$this->command = $value;
						break;
					case "status" :
						$count++;
						$this->status = $value;
						break;
					case "action" :
						$count++;
						$this->actions[] = $value;
						break;
					default:
						break;
				}
			}
		}

		if (! $count > 0 ) {
			$this = "";
			return;
		}
	}

	function _emptyService() {
		; // nothing
	}

	function getStatus() {
		return $this->status;
	}

	function getPID() {
		return $this->pid;
	}

	function getCommand() {
		return $this->command;
	}

	function hasAction($action) {
		if (count(array_intersect($this->actions, array($action))) == 1) {
			$retval = true;
		} else {
			$retval = false;
		}

		if ($this->name == 'ldap' && $action == 'stop') {
			$retval = false;
		}

		return $retval;
	}

	function setAttr($attr, $val) {
		switch ($attr) {
			case "service":
				$this->name = $val;
				break;
			case "pid":
				$this->pid = $val;
				break;
			case "command":
				$this->command = $val;
				break;
			case "action":
				$this->action = $val;
				break;
			case "status":
				$this->status = $val;
				break;
			default:
				break;
		}
	}

	function action($action) {
		if (! $this->hasAction($action)) {
			$GLOBALS['LAST_ERROR'][] = 'Action ' . $action .
				' is not supported';
			$GLOBALS['LAST_ERROR_EXIT'] = 1;
			return false;
		}

		$cmd = new Command('msgserviceaction');
		$cmd->add_arg('--service', $this->name);
		$cmd->add_arg('--action', $action);
		$cmd->run();

		if ($cmd->getExitVal() == 0) {
			return true;
		}
		$GLOBALS['LAST_ERROR'] = $cmd->getErrors();
		$GLOBALS['LAST_ERROR'][] = "\non command:\n" . $cmd->concat();
		$GLOBALS['LAST_ERROR_EXIT'] = $cmd->getExitVal();
		return false;
	}

	function comparisonFunctionName($field, $direction) {
		switch($field) {
			case "service":
				$cmpFunction = "_service_byService";
				break;
			case "status":
				$cmpFunction = "_service_byStatus";
				break;
			default:
				$cmpFunction = "_service_byService";
				break;
		}

		switch($direction) {
			case "a":
				$cmpFunction .= "Ascend";
				break;
			case "d":
				$cmpFunction .= "Decend";
				break;
			default:
				$cmpFunction .= "Ascend";
				break;
		}
		return $cmpFunction;
	}
}

/*
 * Ideally these comparison functions would be encapsulated in the 
 * User class as class methods. Can't work out how to pass a class
 * method to the usort() function yet.
 */
function _service_byServiceAscend($left, $right) {
	return(strcasecmp($left->name, $right->name));
}

function _service_byServiceDecend($left, $right) {
	return(-1 * (_service_byServiceAscend($left, $right)));
}

function _service_byStatusAscend($left, $right) {
	return(strcasecmp($left->status, $right->status));
}

function _service_byStatusDecend($left, $right) {
	return(-1 * (_service_byStatusAscend($left, $right)));
}

?>
