<?php 
/*
 * Copyright (C) 2001 Caldera International, Inc.  All Rights Reserved.
 */

class Table {
	var $columns;
	var $sortedColumn;
	var $offset;
	var $id;
	var $rows;
	var $templates;
	var $linkactionfn;

	function Table($id = "") {
		$this->rows = $GLOBALS['SEARCH_NUM_ROWS'];
		$this->offset = 0;
		$this->id = $id;
		$this->initTemplates();
		$this->linkactionfn = 'Go';
	}

	function addDataColumn($heading, $sort, $sortAttrName, $action="") {
		$this->columns[] = new DataColumn($heading, $sort, $sortAttrName, $action);
	}

	function addDisplayColumn($heading, $action="") {
		$this->columns[] = new DisplayColumn($heading, $action);
	}

	function countColumns() {
		return count($this->columns);
	}

	function getColumnHeading($index) {
		return $this->columns[$index]->getHeading();
	}

	function getColumnSortDir($index) {
		return $this->columns[$index]->getSort();
	}

	function getSortAttrName($index) {
		return $this->columns[$index]->attrName;
		return $this->columns[$index]->getAttrName();
	}

	function setSortedColumn($index) {
		// match string values to 0
		if ($index == 0) { $index = 0; }

		// Set table column to be sorted on (index)
		if ($index < 0 || $index >= $this->countColumns()) {
			$index = $this->firstSortedColumn();
		} elseif (! $this->columns[$index]->isSorted()) {
			$index = $this->firstSortedColumn();
		}
		$this->sortedColumn = $index;
	}

	function getSortedColumn() {
		return $this->sortedColumn;
	}

	function isColumnSorted($index) {
		return $this->columns[$index]->isSorted();
	}

	function setSortDir($index, $dir) {
		if ($dir != Column::DECEND()) { $dir = Column::ASCEND(); }
		$this->columns[$index]->setSort($dir);
	}

	function resetOffset() {
		$this->offset = 0;
	}

	function setOffset($offset, $max) {
		if ($offset <= 0) {
			// lower bound
			// equality test to match strings too
			$this->offset = 0;
		} elseif ($offset > $max) {
			// upper bound
			$this->offset = 0;
		} else {
			$this->offset = $offset;
		}

		return $this->offset;
	}
		

	function getOffset() {
		return $this->offset;
	}
		
	function setId($id) {
		$this->id = $id;
	}

	function getId() {
		return $this->id;
	}

	function setRows($rows) {
		if ($rows > 0 && $rows < $GLOBALS['MAX_ROWS_EVER']) {	
			$this->rows = $rows;
		} else {
			$this->rows = $GLOBALS['SEARCH_NUM_ROWS'];
		}
	}

	function getAction($col) {
		return $this->columns[$col]->getAction();
	}

	function isLink($col) {
		return $this->columns[$col]->isLink();
	}

	function getRows() {
		return $this->rows;
	}

	function firstSortedColumn() {
		for ($i = 0; $i < count($this->columns); $i++) {
			if ($this->columns[$i]->isSorted()) {
				return $i;
			}
		}
		return "";
	}

	function setParams($varArray, $maxOffset) {

		// Sort parameters
		if (isset($varArray['col']) && isset($varArray['dir'])) {
			$this->setSortedColumn($varArray['col']);
			$this->setSortDir($this->getSortedColumn(),
					  $varArray['dir']);
			$dir = $varArray['dir'];
		}

		// Offset parameter
		if (isset($varArray['offset'])) {
			$this->setOffset($varArray['offset'], $maxOffset);
		} else {
			// check the existing offset didn't go out of range
			$this->offset = min($this->offset, $maxOffset);
		}


		// Row parameter
		if (isset($varArray['rows'])) {
			$this->setRows($rows);
		}

	}

	/**********************************************************************
	 * Implemented templates are:
	 * headingRow, headingCell, headingCellSorted, dataRow, dataCell,
	 * dataVal, dataLnk, dataEmpty.
         * sortImgAscendActive, sortImgDecendActive, sortImgAscendInactive,
	 * sortImgDecendInactive, dataRowEven, dataRowOdd are optional.
	 *********************************************************************/
	function initTemplates() {
		$this->templates['headingRow']  = '<tr>%%VALUE%%</tr>';
		$this->templates['headingCell'] = '<th>%%VALUE%%</th>';
		$this->templates['dataRow']     = '<tr>%%VALUE%%</tr>';
		$this->templates['dataEmpty']   = '<tr><td>%%VALUE%%</td></tr>';
		$this->templates['dataCell']    = '<td>%%VALUE%%</td>';
		$this->templates['dataVal']     = '<a href="#" onClick="%%ACTION%%">$_VALUE</a>';
		$this->templates['dataValLnk']  = '%%VALUE%%';
		$this->templates['headingCellSorted'] = '<th>%%VALUE%%</th>';
		$this->templates['pageLink']    = '<a href="#" onClick="return %%ACTIONFN%%(\'%%ACTIONARGS%%\')">%%VALUE%%</a>';
		$this->templates['sortLink']    = '<a href="#" onClick="return %%ACTIONFN%%(\'%%ACTIONARGS%%\')">%%VALUE%%</a>';
	}

	function setTemplate($tmpl, $html) {
		$this->templates[$tmpl] = $html;
	}

	function getTemplate($tmpl) {
		return $this->templates[$tmpl];
	}


	function getSortImg($index) {
	
		// set image and flip dir
		$dir = $this->getColumnSortDir($index);
		if ($dir == Column::ASCEND()) {
			$image = $this->getTemplate('sortImgAscendInactive');
			if ($this->getSortedColumn() == $index) {
				$image = $this->getTemplate('sortImgAscendActive');
			}
		} elseif ($dir == Column::DECEND()) {
			$image = $this->getTemplate('sortImgDecendInactive');
			if ($this->getSortedColumn() == $index) {
				$image = $this->getTemplate('sortImgDecendActive');
			}
		}
		return $image;
	}

	function getSortNextDir($index) {
		// set image and flip dir
		$dir = $this->getColumnSortDir($index);
		if ($dir == Column::ASCEND()) {
			if ($this->getSortedColumn() == $index) {
				$dir = Column::DECEND();
			}
		} elseif ($dir == Column::DECEND()) {
			if ($this->getSortedColumn() == $index) {
				$dir = Column::ASCEND();
			}
		}
		return $dir;
	}
			
	function formatSortArgs($file, $index, $dir, $serial, $extra = '') {
		$retval = $file . 
			'?col=' . $index .
			'&dir=' . $dir .
			'&serial=' . $serial;

		if ($extra != '') {
			$retval .= $extra;
		}

		return $retval;
	}

	function isActive($varArray, $id) {
		global $session_tableObj;

		if ($session_tableObj != "") {
			if ($session_tableObj->getId() == $id) {
				return true;
			}
		}
		return false;
	}

	function getActive() {
		global $session_tableObj;

		if ($session_tableObj == "") {
			return "";
		}
		return $session_tableObj;
	}

	function save() {
		global $session_tableObj;
		
		$session_tableObj = $this;
	}


	function getColumnValue($listidx, $colidx) {
	}

	function getPrevOffset($count) {
		return max(0, $this->offset - $this->rows);
	}

	function getNextOffset($count) {
		return min($count, $this->offset + $this->rows);
	}

	function getPrevNEntries($count) {
		return min($this->rows, $this->offset);
	}

	function getNextNEntries($count) {
		return min($this->rows, $count - $this->offset - $this->rows);
	}

	// deprecated function... left for the logic
	function getPositionInfo($count) {
		$start = $this->offset + 1;
		$end   = $this->offset + $this->rows;
		$end   = min($end, $count);
		$retval = "Entries $start-$end  : ";
		$retval .= $count . " Users Found ";
	}

	// deprecated function... left for the logic
	function getPrevInfo() {
		global $_SELF, $_SERIAL;

		if ($this->offset > 0) {
			$offset = $this->getPrevOffset($count);
			$entries = $this->getPrevNEntries($count);

			$action = $this->linkactionfn;
			$actionargs = $_SELF . 
				  '?offset=' . $offset .
				  '&serial=' . $_SERIAL;
			$linkval = 'previous ' . $entries . ' entries';

			$t = $this->templates['pageLink'];
			$t = ereg_replace('%%ACTIONFN%%', $action, $t);
			$t = ereg_replace('%%ACTIONARGS%%', $actionargs, $t);
			$t = ereg_replace('%%VALUE%%', $linkval, $t);
			$retval = $t;
		} else {
			$retval = '';
		}
		return $retval;
	}

	function getNextInfo($count) {
		global $_SELF, $_SERIAL;

		$end   = $this->offset + $this->rows;
		if ($end < $count) {
			$offset  = $this->getNextOffset($count);
			$entries = $this->getNextNEntries($count);

			$action = $this->linkactionfn;
			$actionargs = $_SELF . 
				  '?offset=' . $offset .
				  '&serial=' . $_SERIAL;
			$linkval = 'next ' . $entries . ' entries';

			$t = $this->templates['pageLink'];
			$t = ereg_replace('%%ACTIONFN%%', $action, $t);
			$t = ereg_replace('%%ACTIONARGS%%', $actionargs, $t);
			$t = ereg_replace('%%VALUE%%', $linkval, $t);
			$retval = $t;
		} else {
			$retval = '';
		}
		return $retval;
	}
}

// Private Class for Table
class Column {
	// Class constants -- apparently they don't work substiute functions below
	var $ASCEND = a;
	var $DECEND = d;
	var $NOSORT = n;

	// fields
	var $heading;	    // Text of column heading
	var $sort;	    // $ASCEND | $DECEND | $NOSORT
	var $action;

	function ASCEND() {
		return a;
	}

	function DECEND() {
		return d;
	}

	function NOSORT() {
		return n;
	}

	function Column($heading, $sort, $action) {
		$this->heading = $heading;
		$this->sort = $sort;
		$this->action= $action;
	}

	function getHeading() {
		return $this->heading;
	}

	function setHeading($value) {
		$this->heading=$value;
	}

	function getSort() {
		return $this->sort;
	}

	function setSort($value) {
		if ($this->sort == $this->ASCEND ||
		    $this->sort == $this->DECEND) {
			$this->sort=$value;
		} else {
			$this->sort=$this->NOSORT;
		}
	}

	function isSorted() {
		if ($this->sort == $this->ASCEND ||
		    $this->sort == $this->DECEND) {
			return true;
		} else {
			return false;
		}
	}


	function setAction($action) {
		$this->action = $action;
	}

	function getAction() {
		return $this->action;
	}

	function isLink() {
		if ($this->action != "") {
			return true;
		}
		return false;
	}
	
}

// Private Class for Table
class DataColumn extends Column {
	var $attrName;

	function DataColumn($heading, $sort, $attrName, $action="") {
		$this->attrName = $attrName;
		$this->Column($heading, $sort, $action);
	}


	function setAttrName($attr) {
		$this->attrName = $attr;
	}

	function getAttrName() {
		return $this->attrName;
	}


}

// Private Class for Table
class DisplayColumn extends Column {
	function DisplayColumn($heading, $action="") {
		$this->Column($heading, $this->$NOSORT, $action);
	}
}

?>
