<?php
/*
 * Copyright (C) 2001 Caldera International, Inc.  All Rights Reserved.
 */

class User {
	var $errorMsgs;

	/* Properties */
	// sv = single valued, mv = multivalued
	var $uid; 	// sv, User Id (inbox name), *required*
	var $firstname;	// sv, First Name
	var $lastname;	// sv, Surname, *required*
	var $fullname;	// sv, Common Name (usually firstname + lastname)
	var $mail;	// sv, Email address (with domain), *required*
	var $pw;	// sv, Password, *required*
	var $wphone;	// sv, Work phone number
	var $mphone;	// sv, Mobile phone number
	var $hphone;	// sv, Home phone number
	var $pager;	// sv, Pager number
	var $fax;	// sv, Fax number
	var $title;	// sv, Business Title
	var $location;  // sv, Physical office delievery location
	var $altmail;	// sv, Alternate email address
	var $forward;	// sv, Email address to forward mail to
	var $vacation;	// sv, boolean vacation notification active or not
	var $vsubject;	// sv, Subject line for vacation notification
	var $vmessage;	// sv, Message body for vacation notification
	var $spam;	// sv, boolean spam filtering active or not

	/* Methods */

	function User() {
		global $MSG_ID, $MSG_NAME, $MSG_STATUS;

		$numArgs = func_num_args();

		switch ($numArgs) {
		  case 0:
			$this->_emptyUser();
			break;
		  case 1:
			$arg1 = func_get_arg(0);
			if (is_array($arg1)) {
				$ret = $this->_createUser($arg1);
				if (! $ret) {
					$this = "";
				}
			} else {
				$this->_fetchUser($arg1);
			}
			break;
		  case 2:
			$arg1 = func_get_arg(0); // uid or mail address
			$arg2 = func_get_arg(2); // array of attributes
			$this->_fetchUser($arg1, $arg2);
		  case 3:
			$arg1 = func_get_arg(0);
			$arg2 = func_get_arg(1);
			$arg3 = func_get_arg(2);
			$this->User3($arg1, $arg2, $arg3);
			break;
		  default:
			$this = "";
			break;
		}

	}

	function _fetchUser($uid, $argArray="") {

		if ($uid == "") {
			$this = "";
			return;
		}

		/* No longer see where this is used
		if ($argArray != "") {
			$attrs = "";
			foreach ($argArray as $attr) $attrs .= _attrOption($attr);
		} else {
			$attrs = " --all";
		}
		*/
		$attrs = '--all';
		$cmd = new Command('msguserlist');
		if (strpos($uid, '@') === false) {
			$cmd->add_arg('--uid', $uid);
		} else {
			$cmd->add_arg('--mail', $uid);
		}
		$cmd->add_arg($attrs);

		$cmd->run();

		if ($cmd->getExitVal() !== 0) {
			$GLOBALS['LAST_ERROR'] = $cmd->getErrors();
			$GLOBALS['LAST_ERROR'][] = "\non command:\n" . $cmd->concat();
			$GLOBALS['LAST_ERROR_EXIT'] = $cmd->getExitVal();
			$this = "";
			return;
		}

		$lines = $cmd->getOutput();

		if ($lines == "") {
			$this = "";
			return;
		}

		$count = 0;
		$lines = array_reverse($lines);
		while ($lines) {
			$line = trim(array_pop($lines));

			if ($line != "") {
				list($attr, $value) = split(":", $line, 2);

				switch ($attr) {
					case "uid" :
						$count++;
						$this->uid = strtolower($value);
						break;
					case "firstname" :
						$count++;
						$this->firstname = $value;
						break;
					case "lastname" :
						$count++;
						$this->lastname = $value;
						break;
					case "fullname" :
						$count++;
						$this->fullname = $value;
						break;
					case "mail" :
						$count++;
						$this->mail = $value;
						break;
					case "businessphone" :
						$count++;
						$this->wphone = $value;
						break;
					case "mobile" :
						$count++;
						$this->mphone = $value;
						break;
					case "homephone" :
						$count++;
						$this->hphone = $value;
						break;
					case "pager" :
						$count++;
						$this->pager = $value;
						break;
					case "fax" :
						$count++;
						$this->fax = $value;
						break;
					case "title" :
						$count++;
						$this->title = $value;
						break;
					case "mailstop" :
						$count++;
						$this->location = $value;
						break;
					case "alternatemail" :
						$count++;
						$this->altmail = $value;
						break;
					case "forward" :
						$count++;
						$this->forward = $value;
						break;
					case "vacationenabled" :
						$count++;
						if (strcasecmp($value, 'true') == 0) {
							$this->vacation = true;
						} else {
							$this->vacation = false;
						}
						break;
					case "vacationsubject" :
						$count++;
						$this->vsubject = $value;
						break;
					case "vacationmessage" :
						$count++;
						while (true) {
						  if (substr($value, -1) == '\\') {
						    $this->vmessage .= substr($value, 0, -1) . "\n";
						    $value = array_pop($lines);
						  } else {
						    $this->vmessage .= $value;
						    break;
						  }
						}
						break;
					case "spamfilterenabled" :
						$count++;
						if (strcasecmp($value, 'true') == 0) {
							$this->spam = true;
						} else {
							$this->spam = false;
						}
						break;
					default:
						break;
				}
			}
		}

		if (! $count > 0 ) {
			$this = "";
			return;
		}
	}

	function _emptyUser() {
		; // nothing
	}

	function _createUser($varArray) {
		global $session_domain;

		// Can only be run by admin
		if (! isAdmin()) {
			$LAST_ERROR[] = 'You do not have create permission';
			return false;
		}

		$cmd = new Command('msgusercreate');
		$cmd->add_arg('--domain', $session_domain);

		while (list ($attr, $val) = each ($varArray)) {

			$val = trim($val);

			if ($val == '')  continue;

			$attr = str_replace('_', '', $attr);

			if (! $this->_validate($attr, $val)) continue;

			switch ($attr) {
				case 'uid' :
				    $val = strtolower($val);
				    $this->uid = $val;
				    break;
				case 'mail' :
				case 'altmail' :
				    if ($val != '') {
				    	$val = $val. '@' . $session_domain;
				    }
				    break;
				case 'firstname' :
				    $this->firstname = $val;
				    break;
				case 'lastname' :
				    $this->lastname = $val;
				    break;
				case 'fullname' :
				    $this->fullname = $val;
				    break;
				default:
				    break;
			}

			$opt = $this->_getOption($attr);
			if ($opt != "") {
				$cmd->add_arg($opt, $val);
			}
		}

		if ($this->fullname === '') {
			$opt = $this->_getOption('fullname');
			$val = $this->firstname . ' ' . $this->lastname;
			if ($opt != "") {
				$cmd->add_arg($opt, $val);
			}
		}

		$cmd->run();

		if ($cmd->getExitVal() === 0) {
			$this->_fetchUser($this->uid);
			return true;
		}
		$GLOBALS['LAST_ERROR'] = $cmd->getErrors();
		$GLOBALS['LAST_ERROR'][] = "\non command:\n" . $cmd->concat();
		$GLOBALS['LAST_ERROR_EXIT'] = $cmd->getExitVal();
		return false;
	}

	function _validate($attr, $value) {
		return true;
	}

	function _getOption($attr) {
		switch ($attr) {
			case "_uid" :
			case "uid" :
				$retval = "--uid";
				break;
			case "_lastname" :
			case "lastname" :
				$retval = "--lastname";
				break;
			case "_firstname" :
			case "firstname" :
				$retval = "--firstname";
				break;
			case "_fullname" :
			case "fullname" :
				$retval = "--fullname";
				break;
			case "_mail" :
			case "mail" :
				$retval = "--mail";
				break;
			case "_hphone" :
			case "hphone" :
				$retval = "--homephone";
				break;
			case "_pager" :
			case "pager" :
				$retval = "--pager";
				break;
			case "_mphone" :
			case "mphone" :
				$retval = "--mobile";
				break;
			case "_wphone" :
			case "wphone" :
				$retval = "--businessphone";
				break;
			case "_fax" :
			case "fax" :
				$retval = "--fax";
				break;
			case "_title" :
			case "title" :
				$retval = "--title";
				break;
			case "_location" :
			case "location" :
				$retval = "--mailstop";
				break;
			case "_altmail" :
			case "altmail" :
				$retval = "--alternatemail";
				break;
			case "_forward" :
			case "forward" :
				$retval = "--forward";
				break;
			case "_vacation" :
			case "vacation" :
				$retval = "--vacationenabled";
				break;
			case "_vsubject" :
			case "vsubject" :
				$retval = "--vacationsubject";
				break;
			case "_vmessage" :
			case "vmessage" :
				$retval = "--vacationmessage";
				break;
			case "_spam" :
			case "spam" :
				$retval = "--spamfilterenabled";
				break;
			default:
				$retval = "";
		}

		return $retval;
	}

	function formExport() {
		$attrs = get_object_vars($this);
		foreach ($attrs as $attr => $val) {
			$GLOBALS["_$attr"] = $val;
		}
	}

	function setAttr($attr, $val) {
		switch ($attr) {
			case "uid":
				$this->uid = $val;
				break;
			case "fullname":
				$this->fullname = $val;
				break;
			case "mail":
				$this->mail = $val;
				break;
			default:
				break;
		}
	}

	function getAttrs(&$attrArray) {
		$attrArray = array('uid', 'fullname', 'mail');
		return;
	}
	

	function setUID($value) {
		$this->uid = $value;
	}

	function getUID() {
		if ($this->uid == "") return "(empty)";
		return $this->uid;
	}

	function setFullName($value) {
		$this->fullname = $value;
	}

	function getFullName() {
		if ($this->fullname == "") return "(empty)";
		return $this->fullname;
	}

	function setMail($value) {
		$this->mail = $value;
	}

	function getMail() {
		if ($this->mail == "") return "(empty)";
		return $this->mail;
	}

	function setidLabel($value) {
		$this->idLabel = $value;
	}

	function delete($uid) {
		if (Authenticate::isAdmin($uid)) {
			$GLOBALS['LAST_ERROR'][] = getMessage('MSG_CANT_DELETE_ADMIN');
			return false;
		}
	
		$cmd = new Command('msguserdelete');
		$cmd->add_arg('--uid', $uid);
		$cmd->add_arg('--recurse');
		$cmd->run();

		if ($cmd->getExitVal() == 0) {
			return true;
		}

		$GLOBALS['LAST_ERROR'] = $cmd->getErrors();
		$GLOBALS['LAST_ERROR'][] = "\non command:\n" . $cmd->concat();
		$GLOBALS['LAST_ERROR_EXIT'] = $cmd->getExitVal();
		return false;
	}

	function comparisonFunctionName($field, $direction) {
		switch($field) {
			case "uid":
				$cmpFunction = "byUID";
				break;
			case "name":
				$cmpFunction = "byFullName";
				break;
			case "mail":
				$cmpFunction = "byMail";
				break;
			default:
				$cmpFunction = "byUID";
				break;
		}

		switch($direction) {
			case "a":
				$cmpFunction .= "Ascend";
				break;
			case "d":
				$cmpFunction .= "Decend";
				break;
			default:
				$cmpFunction .= "Acend";
				break;
		}
		return $cmpFunction;
	}

	function match($expr, $attr="") {
		if ($attr != "") {
			$retval = ereg($expression, $this->$attr);
		} else {
			if (	eregi($expr, $this->uid)     ||
				eregi($expr, $this->fullname)   ||
				eregi($expr, $this->mail) ||
				eregi($expr, $this->uid)   ||
				eregi($expr, $this->uid)   ) {
				$retval = true;
			} else {
				$retval = false;
			}
		}
		return $retval;
	}

	function setForward($varArray) {
		if (! isset($varArray['_forward'])) {
			return false;
		}
		$value = $varArray['_forward'];
		return $this->modify(array('_forward' => $value));
	}

	function modify($varArray) {
		global $session_domain, $session_perms, $LAST_ERROR;

		// If not admin... can only modify self.
		// Ensure modify priv has been granted.
		if (! isAdmin()) {
			if ($this->uid == $session_user) {
				$LAST_ERROR[] = 'You do not have modify permission';
				return false;
			}

			if (! $session_perms->canChangeProfile()) {
				$LAST_ERROR[] = 'You do not have modify permission';
				return false;
			}
		}

		$cmd = new Command('msgusermod');
		$cmd->add_arg('--uid', $this->uid);
		$current = get_object_vars($this);

		while (list ($attr, $val) = each ($varArray)) {
			$skip = false;

			$val = trim($val);

			$attr = str_replace("_", "", $attr);

			if (! $this->_validate($attr, $val)) continue;

			if ($current[$attr] == $val) continue;

			switch ($attr) {
				case "uid" :
				    continue; // uid cannot be modified
				case "mail" :
				case "altmail" :
				    if (! isAdmin()) break;

				    if ($val != '') {
				    	$val = $val. '@' . $session_domain;
				    }

				    if ($current[$attr] == $val) continue 2;

				    break;
				case "vacation":
				case "spam":
				    $val = strtoupper($val);
				    break;
				case "vmessage":
				    $val = ereg_replace("\r", '', $val);
				    break;
				default: 
				    break;
			}

			if (! $skip) {
				$opt = $this->_getOption($attr);
				if ($opt != "") {
					$cmd->add_arg($opt, $val);
				}
			}
		}

		$cmd->run();

		if ($cmd->getExitVal() == 0) {
			$this = new User($this->uid);
			return true;
		}
		$GLOBALS['LAST_ERROR'] = $cmd->getErrors();
		$GLOBALS['LAST_ERROR'][] = "\non command:\n" . $cmd->concat();
		$GLOBALS['LAST_ERROR_EXIT'] = $cmd->getExitVal();
		return false;
	}

	function isRequired($attr) {
		switch ($attr) {
			case 'uid':
			case 'mail':
			case 'pw':
				$retval = true;
				break;
			case 'fullname':
			case 'firstname':
			case 'lastname':
			case 'wphone':
			case 'mphone':
			case 'hphone':
			case 'pager':
			case 'fax':
			case 'title':
			case 'location':
			case 'altmail':
			case 'forward':
			case 'vacation':
			case 'vsubject':
			case 'vmessage':
			case 'spam':
			default:
				$retval = false;
				break;
		}
		return $retval;
	}

	function shortmail() {
		$pos = strpos($this->mail, '@');
		if ($pos === false) {
			$retval = $this->mail;
		} else {
			$retval = substr($this->mail, 0, $pos);
		}
		
		return $retval;
	}

}

/*
 * Ideally these comparison functions would be encapsulated in the 
 * User class as class methods. Can't work out how to pass a class
 * method to the usort() function yet.
 */
function byUIDAscend($left, $right) {
	return(strcasecmp($left->uid,$right->uid));
}

function byUIDDecend($left, $right) {
	return(-1 * (byUIDAscend($left, $right)));
}

function byFullNameAscend($left, $right) {
	return(strcasecmp($left->fullname,$right->fullname));
}

function byFullNameDecend($left, $right) {
	return(-1 * (byFullNameAscend($left, $right)));
}

function byMailAscend($left, $right) {
	return(strcasecmp($left->mail,$right->mail));
}

function byMailDecend($left, $right) {
	return(-1 * (byMailAscend($left, $right)));
}

?>
